# Generated by ariadne-codegen

from .add_pipeline_recipient import (
    AddPipelineRecipient,
    AddPipelineRecipientAddPipelineRecipient,
)
from .add_webapp_to_favorites import (
    AddWebappToFavorites,
    AddWebappToFavoritesAddToFavorites,
)
from .archive_workspace import ArchiveWorkspace, ArchiveWorkspaceArchiveWorkspace
from .base_client import BaseClient
from .base_model import BaseModel, Upload
from .client import Client
from .create_connection import (
    CreateConnection,
    CreateConnectionCreateConnection,
    CreateConnectionCreateConnectionConnection,
)
from .create_dataset import (
    CreateDataset,
    CreateDatasetCreateDataset,
    CreateDatasetCreateDatasetDataset,
)
from .create_pipeline import (
    CreatePipeline,
    CreatePipelineCreatePipeline,
    CreatePipelineCreatePipelinePipeline,
)
from .create_pipeline_from_template_version import (
    CreatePipelineFromTemplateVersion,
    CreatePipelineFromTemplateVersionCreatePipelineFromTemplateVersion,
    CreatePipelineFromTemplateVersionCreatePipelineFromTemplateVersionPipeline,
)
from .create_pipeline_template_version import (
    CreatePipelineTemplateVersion,
    CreatePipelineTemplateVersionCreatePipelineTemplateVersion,
    CreatePipelineTemplateVersionCreatePipelineTemplateVersionPipelineTemplate,
)
from .create_webapp import (
    CreateWebapp,
    CreateWebappCreateWebapp,
    CreateWebappCreateWebappWebapp,
)
from .create_workspace import (
    CreateWorkspace,
    CreateWorkspaceCreateWorkspace,
    CreateWorkspaceCreateWorkspaceWorkspace,
    CreateWorkspaceCreateWorkspaceWorkspaceCountries,
)
from .dataset import (
    Dataset,
    DatasetDataset,
    DatasetDatasetCreatedBy,
    DatasetDatasetPermissions,
    DatasetDatasetVersions,
    DatasetDatasetVersionsItems,
    DatasetDatasetVersionsItemsCreatedBy,
    DatasetDatasetWorkspace,
)
from .datasets import (
    Datasets,
    DatasetsDatasets,
    DatasetsDatasetsItems,
    DatasetsDatasetsItemsCreatedBy,
    DatasetsDatasetsItemsPermissions,
)
from .delete_connection import DeleteConnection, DeleteConnectionDeleteConnection
from .delete_dataset import DeleteDataset, DeleteDatasetDeleteDataset
from .delete_pipeline import DeletePipeline, DeletePipelineDeletePipeline
from .delete_pipeline_template import (
    DeletePipelineTemplate,
    DeletePipelineTemplateDeletePipelineTemplate,
)
from .delete_pipeline_version import (
    DeletePipelineVersion,
    DeletePipelineVersionDeletePipelineVersion,
)
from .delete_webapp import DeleteWebapp, DeleteWebappDeleteWebapp
from .enums import (
    AccessmodAccessibilityAnalysisAlgorithm,
    AccessmodAccessRequestStatus,
    AccessmodAnalysisStatus,
    AccessmodAnalysisType,
    AccessmodFilesetFormat,
    AccessmodFilesetMode,
    AccessmodFilesetRoleCode,
    AccessmodFilesetStatus,
    AccessmodProjectOrder,
    AddToFavoritesError,
    ApproveAccessmodAccessRequestError,
    ArchiveWorkspaceError,
    BucketObjectType,
    ConnectionType,
    CreateAccessmodAccessibilityAnalysisError,
    CreateAccessmodFileError,
    CreateAccessmodFilesetError,
    CreateAccessmodProjectError,
    CreateAccessmodProjectMemberError,
    CreateAccessmodZonalStatisticsError,
    CreateBucketFolderError,
    CreateConnectionError,
    CreateDatasetError,
    CreateDatasetVersionError,
    CreateDatasetVersionFileError,
    CreateMembershipError,
    CreatePipelineFromTemplateVersionError,
    CreatePipelineTemplateVersionError,
    CreateTeamError,
    CreateTemplateVersionPermissionReason,
    CreateWebappError,
    CreateWorkspaceError,
    DAGRunOrderBy,
    DAGRunStatus,
    DAGRunTrigger,
    DeclineWorkspaceInvitationError,
    DeleteAccessmodAnalysisError,
    DeleteAccessmodFilesetError,
    DeleteAccessmodProjectError,
    DeleteAccessmodProjectMemberError,
    DeleteBucketObjectError,
    DeleteConnectionError,
    DeleteDatasetError,
    DeleteDatasetLinkError,
    DeleteDatasetVersionError,
    DeleteMembershipError,
    DeleteMetadataAttributeError,
    DeletePipelineVersionError,
    DeleteTeamError,
    DeleteTemplateVersionError,
    DeleteWebappError,
    DeleteWorkspaceDatabaseTableError,
    DeleteWorkspaceError,
    DeleteWorkspaceInvitationError,
    DeleteWorkspaceMemberError,
    DenyAccessmodAccessRequestError,
    DHIS2ConnectionError,
    DHIS2ConnectionStatus,
    DHIS2MetadataType,
    DisableTwoFactorError,
    EnableTwoFactorError,
    FileSampleStatus,
    FileType,
    GenerateChallengeError,
    GenerateNewDatabasePasswordError,
    GeneratePipelineWebhookUrlError,
    GenerateWorkspaceTokenError,
    IASOConnectionError,
    IASOMetadataType,
    InviteWorkspaceMembershipError,
    JoinWorkspaceError,
    LaunchAccessmodAnalysisError,
    LaunchNotebookServerError,
    LinkDatasetError,
    LoginError,
    MembershipRole,
    MessagePriority,
    OrderByDirection,
    ParameterType,
    ParameterWidget,
    PermissionMode,
    PinDatasetError,
    PipelineError,
    PipelineNotificationLevel,
    PipelineRecipientError,
    PipelineRunOrderBy,
    PipelineRunStatus,
    PipelineRunTrigger,
    PipelineTemplateError,
    PipelineType,
    PrepareObjectDownloadError,
    PrepareObjectUploadError,
    PrepareVersionFileDownloadError,
    RegisterError,
    RemoveFromFavoritesError,
    RequestAccessmodAccessError,
    ResendWorkspaceInvitationError,
    RunDAGError,
    SetDAGRunFavoriteError,
    SetMetadataAttributeError,
    SetPasswordError,
    UpdateAccessmodAccessibilityAnalysisError,
    UpdateAccessmodFilesetError,
    UpdateAccessmodProjectError,
    UpdateAccessmodProjectMemberError,
    UpdateAccessmodZonalStatisticsError,
    UpdateConnectionError,
    UpdateDAGError,
    UpdateDatasetError,
    UpdateDatasetVersionError,
    UpdateMembershipError,
    UpdatePipelineError,
    UpdatePipelineVersionError,
    UpdateTeamError,
    UpdateTemplateError,
    UpdateTemplateVersionError,
    UpdateUserError,
    UpdateWebappError,
    UpdateWorkspaceError,
    UpdateWorkspaceMemberError,
    UpgradePipelineVersionFromTemplateError,
    VerifyDeviceError,
    WorkspaceInvitationStatus,
    WorkspaceMembershipRole,
)
from .exceptions import (
    GraphQLClientError,
    GraphQLClientGraphQLError,
    GraphQLClientGraphQLMultiError,
    GraphQLClientHttpError,
    GraphQLClientInvalidResponseError,
)
from .get_connection import (
    GetConnection,
    GetConnectionConnectionBySlug,
    GetConnectionConnectionBySlugFields,
)
from .get_users import GetUsers, GetUsersUsers, GetUsersUsersAvatar
from .input_types import (
    AddPipelineOutputInput,
    AddToFavoritesInput,
    ApproveAccessmodAccessRequestInput,
    ArchiveWorkspaceInput,
    ConnectionFieldInput,
    CountryInput,
    CreateAccessmodAccessibilityAnalysisInput,
    CreateAccessmodFileInput,
    CreateAccessmodFilesetInput,
    CreateAccessmodProjectInput,
    CreateAccessmodProjectMemberInput,
    CreateAccessmodZonalStatisticsInput,
    CreateBucketFolderInput,
    CreateConnectionInput,
    CreateDatasetInput,
    CreateDatasetVersionFileInput,
    CreateDatasetVersionInput,
    CreateMembershipInput,
    CreatePipelineFromTemplateVersionInput,
    CreatePipelineInput,
    CreatePipelineRecipientInput,
    CreatePipelineTemplateVersionInput,
    CreateTeamInput,
    CreateWebappInput,
    CreateWorkspaceInput,
    DeclineWorkspaceInvitationInput,
    DeleteAccessmodAnalysisInput,
    DeleteAccessmodFilesetInput,
    DeleteAccessmodProjectInput,
    DeleteAccessmodProjectMemberInput,
    DeleteBucketObjectInput,
    DeleteConnectionInput,
    DeleteDatasetInput,
    DeleteDatasetLinkInput,
    DeleteDatasetVersionInput,
    DeleteMembershipInput,
    DeleteMetadataAttributeInput,
    DeletePipelineInput,
    DeletePipelineRecipientInput,
    DeletePipelineTemplateInput,
    DeletePipelineVersionInput,
    DeleteTeamInput,
    DeleteTemplateVersionInput,
    DeleteWebappInput,
    DeleteWorkspaceDatabaseTableInput,
    DeleteWorkspaceInput,
    DeleteWorkspaceInvitationInput,
    DeleteWorkspaceMemberInput,
    DenyAccessmodAccessRequestInput,
    DisableTwoFactorInput,
    EnableTwoFactorInput,
    GenerateDatasetUploadUrlInput,
    GenerateNewDatabasePasswordInput,
    GeneratePipelineWebhookUrlInput,
    GenerateWorkspaceTokenInput,
    IASOQueryFilterInput,
    InviteWorkspaceMemberInput,
    JoinWorkspaceInput,
    LaunchAccessmodAnalysisInput,
    LaunchNotebookServerInput,
    LinkDatasetInput,
    LoginInput,
    LogPipelineMessageInput,
    OrganizationInput,
    ParameterInput,
    PinDatasetInput,
    PipelineTokenInput,
    PrepareAccessmodFileDownloadInput,
    PrepareAccessmodFilesetVisualizationDownloadInput,
    PrepareAccessmodFileUploadInput,
    PrepareDownloadURLInput,
    PrepareObjectDownloadInput,
    PrepareObjectUploadInput,
    PrepareVersionFileDownloadInput,
    RegisterInput,
    RemoveFromFavoritesInput,
    RequestAccessmodAccessInput,
    ResendWorkspaceInvitationInput,
    ResetPasswordInput,
    RunDAGInput,
    RunPipelineInput,
    SetDAGRunFavoriteInput,
    SetMetadataAttributeInput,
    SetPasswordInput,
    StopPipelineInput,
    UpdateAccessmodAccessibilityAnalysisInput,
    UpdateAccessmodFilesetInput,
    UpdateAccessmodProjectInput,
    UpdateAccessmodProjectMemberInput,
    UpdateAccessmodZonalStatisticsInput,
    UpdateConnectionInput,
    UpdateDAGInput,
    UpdateDatasetInput,
    UpdateDatasetVersionInput,
    UpdateMembershipInput,
    UpdatePipelineInput,
    UpdatePipelineProgressInput,
    UpdatePipelineRecipientInput,
    UpdatePipelineVersionInput,
    UpdateTeamInput,
    UpdateTemplateInput,
    UpdateTemplateVersionInput,
    UpdateUserInput,
    UpdateWebappInput,
    UpdateWorkspaceInput,
    UpdateWorkspaceMemberInput,
    UpgradePipelineVersionFromTemplateInput,
    UploadPipelineInput,
    VerifyDeviceInput,
)
from .invite_workspace_member import (
    InviteWorkspaceMember,
    InviteWorkspaceMemberInviteWorkspaceMember,
    InviteWorkspaceMemberInviteWorkspaceMemberWorkspaceMembership,
)
from .pipeline import (
    Pipeline,
    PipelinePipelineByCode,
    PipelinePipelineByCodeCurrentVersion,
    PipelinePipelineByCodeCurrentVersionUser,
    PipelinePipelineByCodeCurrentVersionUserAvatar,
    PipelinePipelineByCodeNewTemplateVersions,
    PipelinePipelineByCodeRecipients,
    PipelinePipelineByCodeRecipientsUser,
    PipelinePipelineByCodeRuns,
    PipelinePipelineByCodeRunsItems,
    PipelinePipelineByCodeRunsItemsUser,
    PipelinePipelineByCodeSourceTemplate,
)
from .pipelines import (
    Pipelines,
    PipelinesPipelines,
    PipelinesPipelinesItems,
    PipelinesPipelinesItemsCurrentVersion,
    PipelinesPipelinesItemsLastRuns,
    PipelinesPipelinesItemsLastRunsItems,
)
from .remove_webapp_from_favorites import (
    RemoveWebappFromFavorites,
    RemoveWebappFromFavoritesRemoveFromFavorites,
)
from .stop_pipeline import StopPipeline, StopPipelineStopPipeline
from .update_connection import (
    UpdateConnection,
    UpdateConnectionUpdateConnection,
    UpdateConnectionUpdateConnectionConnection,
    UpdateConnectionUpdateConnectionConnectionFields,
)
from .update_dataset import (
    UpdateDataset,
    UpdateDatasetUpdateDataset,
    UpdateDatasetUpdateDatasetDataset,
)
from .update_webapp import UpdateWebapp, UpdateWebappUpdateWebapp
from .update_workspace import (
    UpdateWorkspace,
    UpdateWorkspaceUpdateWorkspace,
    UpdateWorkspaceUpdateWorkspaceWorkspace,
    UpdateWorkspaceUpdateWorkspaceWorkspaceCountries,
)
from .upgrade_pipeline_version_from_template import (
    UpgradePipelineVersionFromTemplate,
    UpgradePipelineVersionFromTemplateUpgradePipelineVersionFromTemplate,
)
from .upload_pipeline import UploadPipeline, UploadPipelineUploadPipeline
from .workspace import (
    Workspace,
    WorkspaceWorkspace,
    WorkspaceWorkspaceCountries,
    WorkspaceWorkspacePermissions,
)
from .workspaces import (
    Workspaces,
    WorkspacesWorkspaces,
    WorkspacesWorkspacesItems,
    WorkspacesWorkspacesItemsCountries,
)

__all__ = [
    "AccessmodAccessRequestStatus",
    "AccessmodAccessibilityAnalysisAlgorithm",
    "AccessmodAnalysisStatus",
    "AccessmodAnalysisType",
    "AccessmodFilesetFormat",
    "AccessmodFilesetMode",
    "AccessmodFilesetRoleCode",
    "AccessmodFilesetStatus",
    "AccessmodProjectOrder",
    "AddPipelineOutputInput",
    "AddPipelineRecipient",
    "AddPipelineRecipientAddPipelineRecipient",
    "AddToFavoritesError",
    "AddToFavoritesInput",
    "AddWebappToFavorites",
    "AddWebappToFavoritesAddToFavorites",
    "ApproveAccessmodAccessRequestError",
    "ApproveAccessmodAccessRequestInput",
    "ArchiveWorkspace",
    "ArchiveWorkspaceArchiveWorkspace",
    "ArchiveWorkspaceError",
    "ArchiveWorkspaceInput",
    "BaseClient",
    "BaseModel",
    "BucketObjectType",
    "Client",
    "ConnectionFieldInput",
    "ConnectionType",
    "CountryInput",
    "CreateAccessmodAccessibilityAnalysisError",
    "CreateAccessmodAccessibilityAnalysisInput",
    "CreateAccessmodFileError",
    "CreateAccessmodFileInput",
    "CreateAccessmodFilesetError",
    "CreateAccessmodFilesetInput",
    "CreateAccessmodProjectError",
    "CreateAccessmodProjectInput",
    "CreateAccessmodProjectMemberError",
    "CreateAccessmodProjectMemberInput",
    "CreateAccessmodZonalStatisticsError",
    "CreateAccessmodZonalStatisticsInput",
    "CreateBucketFolderError",
    "CreateBucketFolderInput",
    "CreateConnection",
    "CreateConnectionCreateConnection",
    "CreateConnectionCreateConnectionConnection",
    "CreateConnectionError",
    "CreateConnectionInput",
    "CreateDataset",
    "CreateDatasetCreateDataset",
    "CreateDatasetCreateDatasetDataset",
    "CreateDatasetError",
    "CreateDatasetInput",
    "CreateDatasetVersionError",
    "CreateDatasetVersionFileError",
    "CreateDatasetVersionFileInput",
    "CreateDatasetVersionInput",
    "CreateMembershipError",
    "CreateMembershipInput",
    "CreatePipeline",
    "CreatePipelineCreatePipeline",
    "CreatePipelineCreatePipelinePipeline",
    "CreatePipelineFromTemplateVersion",
    "CreatePipelineFromTemplateVersionCreatePipelineFromTemplateVersion",
    "CreatePipelineFromTemplateVersionCreatePipelineFromTemplateVersionPipeline",
    "CreatePipelineFromTemplateVersionError",
    "CreatePipelineFromTemplateVersionInput",
    "CreatePipelineInput",
    "CreatePipelineRecipientInput",
    "CreatePipelineTemplateVersion",
    "CreatePipelineTemplateVersionCreatePipelineTemplateVersion",
    "CreatePipelineTemplateVersionCreatePipelineTemplateVersionPipelineTemplate",
    "CreatePipelineTemplateVersionError",
    "CreatePipelineTemplateVersionInput",
    "CreateTeamError",
    "CreateTeamInput",
    "CreateTemplateVersionPermissionReason",
    "CreateWebapp",
    "CreateWebappCreateWebapp",
    "CreateWebappCreateWebappWebapp",
    "CreateWebappError",
    "CreateWebappInput",
    "CreateWorkspace",
    "CreateWorkspaceCreateWorkspace",
    "CreateWorkspaceCreateWorkspaceWorkspace",
    "CreateWorkspaceCreateWorkspaceWorkspaceCountries",
    "CreateWorkspaceError",
    "CreateWorkspaceInput",
    "DAGRunOrderBy",
    "DAGRunStatus",
    "DAGRunTrigger",
    "DHIS2ConnectionError",
    "DHIS2ConnectionStatus",
    "DHIS2MetadataType",
    "Dataset",
    "DatasetDataset",
    "DatasetDatasetCreatedBy",
    "DatasetDatasetPermissions",
    "DatasetDatasetVersions",
    "DatasetDatasetVersionsItems",
    "DatasetDatasetVersionsItemsCreatedBy",
    "DatasetDatasetWorkspace",
    "Datasets",
    "DatasetsDatasets",
    "DatasetsDatasetsItems",
    "DatasetsDatasetsItemsCreatedBy",
    "DatasetsDatasetsItemsPermissions",
    "DeclineWorkspaceInvitationError",
    "DeclineWorkspaceInvitationInput",
    "DeleteAccessmodAnalysisError",
    "DeleteAccessmodAnalysisInput",
    "DeleteAccessmodFilesetError",
    "DeleteAccessmodFilesetInput",
    "DeleteAccessmodProjectError",
    "DeleteAccessmodProjectInput",
    "DeleteAccessmodProjectMemberError",
    "DeleteAccessmodProjectMemberInput",
    "DeleteBucketObjectError",
    "DeleteBucketObjectInput",
    "DeleteConnection",
    "DeleteConnectionDeleteConnection",
    "DeleteConnectionError",
    "DeleteConnectionInput",
    "DeleteDataset",
    "DeleteDatasetDeleteDataset",
    "DeleteDatasetError",
    "DeleteDatasetInput",
    "DeleteDatasetLinkError",
    "DeleteDatasetLinkInput",
    "DeleteDatasetVersionError",
    "DeleteDatasetVersionInput",
    "DeleteMembershipError",
    "DeleteMembershipInput",
    "DeleteMetadataAttributeError",
    "DeleteMetadataAttributeInput",
    "DeletePipeline",
    "DeletePipelineDeletePipeline",
    "DeletePipelineInput",
    "DeletePipelineRecipientInput",
    "DeletePipelineTemplate",
    "DeletePipelineTemplateDeletePipelineTemplate",
    "DeletePipelineTemplateInput",
    "DeletePipelineVersion",
    "DeletePipelineVersionDeletePipelineVersion",
    "DeletePipelineVersionError",
    "DeletePipelineVersionInput",
    "DeleteTeamError",
    "DeleteTeamInput",
    "DeleteTemplateVersionError",
    "DeleteTemplateVersionInput",
    "DeleteWebapp",
    "DeleteWebappDeleteWebapp",
    "DeleteWebappError",
    "DeleteWebappInput",
    "DeleteWorkspaceDatabaseTableError",
    "DeleteWorkspaceDatabaseTableInput",
    "DeleteWorkspaceError",
    "DeleteWorkspaceInput",
    "DeleteWorkspaceInvitationError",
    "DeleteWorkspaceInvitationInput",
    "DeleteWorkspaceMemberError",
    "DeleteWorkspaceMemberInput",
    "DenyAccessmodAccessRequestError",
    "DenyAccessmodAccessRequestInput",
    "DisableTwoFactorError",
    "DisableTwoFactorInput",
    "EnableTwoFactorError",
    "EnableTwoFactorInput",
    "FileSampleStatus",
    "FileType",
    "GenerateChallengeError",
    "GenerateDatasetUploadUrlInput",
    "GenerateNewDatabasePasswordError",
    "GenerateNewDatabasePasswordInput",
    "GeneratePipelineWebhookUrlError",
    "GeneratePipelineWebhookUrlInput",
    "GenerateWorkspaceTokenError",
    "GenerateWorkspaceTokenInput",
    "GetConnection",
    "GetConnectionConnectionBySlug",
    "GetConnectionConnectionBySlugFields",
    "GetUsers",
    "GetUsersUsers",
    "GetUsersUsersAvatar",
    "GraphQLClientError",
    "GraphQLClientGraphQLError",
    "GraphQLClientGraphQLMultiError",
    "GraphQLClientHttpError",
    "GraphQLClientInvalidResponseError",
    "IASOConnectionError",
    "IASOMetadataType",
    "IASOQueryFilterInput",
    "InviteWorkspaceMember",
    "InviteWorkspaceMemberInput",
    "InviteWorkspaceMemberInviteWorkspaceMember",
    "InviteWorkspaceMemberInviteWorkspaceMemberWorkspaceMembership",
    "InviteWorkspaceMembershipError",
    "JoinWorkspaceError",
    "JoinWorkspaceInput",
    "LaunchAccessmodAnalysisError",
    "LaunchAccessmodAnalysisInput",
    "LaunchNotebookServerError",
    "LaunchNotebookServerInput",
    "LinkDatasetError",
    "LinkDatasetInput",
    "LogPipelineMessageInput",
    "LoginError",
    "LoginInput",
    "MembershipRole",
    "MessagePriority",
    "OrderByDirection",
    "OrganizationInput",
    "ParameterInput",
    "ParameterType",
    "ParameterWidget",
    "PermissionMode",
    "PinDatasetError",
    "PinDatasetInput",
    "Pipeline",
    "PipelineError",
    "PipelineNotificationLevel",
    "PipelinePipelineByCode",
    "PipelinePipelineByCodeCurrentVersion",
    "PipelinePipelineByCodeCurrentVersionUser",
    "PipelinePipelineByCodeCurrentVersionUserAvatar",
    "PipelinePipelineByCodeNewTemplateVersions",
    "PipelinePipelineByCodeRecipients",
    "PipelinePipelineByCodeRecipientsUser",
    "PipelinePipelineByCodeRuns",
    "PipelinePipelineByCodeRunsItems",
    "PipelinePipelineByCodeRunsItemsUser",
    "PipelinePipelineByCodeSourceTemplate",
    "PipelineRecipientError",
    "PipelineRunOrderBy",
    "PipelineRunStatus",
    "PipelineRunTrigger",
    "PipelineTemplateError",
    "PipelineTokenInput",
    "PipelineType",
    "Pipelines",
    "PipelinesPipelines",
    "PipelinesPipelinesItems",
    "PipelinesPipelinesItemsCurrentVersion",
    "PipelinesPipelinesItemsLastRuns",
    "PipelinesPipelinesItemsLastRunsItems",
    "PrepareAccessmodFileDownloadInput",
    "PrepareAccessmodFileUploadInput",
    "PrepareAccessmodFilesetVisualizationDownloadInput",
    "PrepareDownloadURLInput",
    "PrepareObjectDownloadError",
    "PrepareObjectDownloadInput",
    "PrepareObjectUploadError",
    "PrepareObjectUploadInput",
    "PrepareVersionFileDownloadError",
    "PrepareVersionFileDownloadInput",
    "RegisterError",
    "RegisterInput",
    "RemoveFromFavoritesError",
    "RemoveFromFavoritesInput",
    "RemoveWebappFromFavorites",
    "RemoveWebappFromFavoritesRemoveFromFavorites",
    "RequestAccessmodAccessError",
    "RequestAccessmodAccessInput",
    "ResendWorkspaceInvitationError",
    "ResendWorkspaceInvitationInput",
    "ResetPasswordInput",
    "RunDAGError",
    "RunDAGInput",
    "RunPipelineInput",
    "SetDAGRunFavoriteError",
    "SetDAGRunFavoriteInput",
    "SetMetadataAttributeError",
    "SetMetadataAttributeInput",
    "SetPasswordError",
    "SetPasswordInput",
    "StopPipeline",
    "StopPipelineInput",
    "StopPipelineStopPipeline",
    "UpdateAccessmodAccessibilityAnalysisError",
    "UpdateAccessmodAccessibilityAnalysisInput",
    "UpdateAccessmodFilesetError",
    "UpdateAccessmodFilesetInput",
    "UpdateAccessmodProjectError",
    "UpdateAccessmodProjectInput",
    "UpdateAccessmodProjectMemberError",
    "UpdateAccessmodProjectMemberInput",
    "UpdateAccessmodZonalStatisticsError",
    "UpdateAccessmodZonalStatisticsInput",
    "UpdateConnection",
    "UpdateConnectionError",
    "UpdateConnectionInput",
    "UpdateConnectionUpdateConnection",
    "UpdateConnectionUpdateConnectionConnection",
    "UpdateConnectionUpdateConnectionConnectionFields",
    "UpdateDAGError",
    "UpdateDAGInput",
    "UpdateDataset",
    "UpdateDatasetError",
    "UpdateDatasetInput",
    "UpdateDatasetUpdateDataset",
    "UpdateDatasetUpdateDatasetDataset",
    "UpdateDatasetVersionError",
    "UpdateDatasetVersionInput",
    "UpdateMembershipError",
    "UpdateMembershipInput",
    "UpdatePipelineError",
    "UpdatePipelineInput",
    "UpdatePipelineProgressInput",
    "UpdatePipelineRecipientInput",
    "UpdatePipelineVersionError",
    "UpdatePipelineVersionInput",
    "UpdateTeamError",
    "UpdateTeamInput",
    "UpdateTemplateError",
    "UpdateTemplateInput",
    "UpdateTemplateVersionError",
    "UpdateTemplateVersionInput",
    "UpdateUserError",
    "UpdateUserInput",
    "UpdateWebapp",
    "UpdateWebappError",
    "UpdateWebappInput",
    "UpdateWebappUpdateWebapp",
    "UpdateWorkspace",
    "UpdateWorkspaceError",
    "UpdateWorkspaceInput",
    "UpdateWorkspaceMemberError",
    "UpdateWorkspaceMemberInput",
    "UpdateWorkspaceUpdateWorkspace",
    "UpdateWorkspaceUpdateWorkspaceWorkspace",
    "UpdateWorkspaceUpdateWorkspaceWorkspaceCountries",
    "UpgradePipelineVersionFromTemplate",
    "UpgradePipelineVersionFromTemplateError",
    "UpgradePipelineVersionFromTemplateInput",
    "UpgradePipelineVersionFromTemplateUpgradePipelineVersionFromTemplate",
    "Upload",
    "UploadPipeline",
    "UploadPipelineInput",
    "UploadPipelineUploadPipeline",
    "VerifyDeviceError",
    "VerifyDeviceInput",
    "Workspace",
    "WorkspaceInvitationStatus",
    "WorkspaceMembershipRole",
    "WorkspaceWorkspace",
    "WorkspaceWorkspaceCountries",
    "WorkspaceWorkspacePermissions",
    "Workspaces",
    "WorkspacesWorkspaces",
    "WorkspacesWorkspacesItems",
    "WorkspacesWorkspacesItemsCountries",
]



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import PipelineType


class GetWorkspacePipelines(BaseModel):
    pipelines: "GetWorkspacePipelinesPipelines"


class GetWorkspacePipelinesPipelines(BaseModel):
    total_pages: int = Field(alias="totalPages")
    items: List["GetWorkspacePipelinesPipelinesItems"]


class GetWorkspacePipelinesPipelinesItems(BaseModel):
    id: Any
    code: str
    name: Optional[str]
    type: PipelineType
    current_version: Optional["GetWorkspacePipelinesPipelinesItemsCurrentVersion"] = (
        Field(alias="currentVersion")
    )


class GetWorkspacePipelinesPipelinesItemsCurrentVersion(BaseModel):
    id: Any
    name: Optional[str]
    version_number: int = Field(alias="versionNumber")


GetWorkspacePipelines.model_rebuild()
GetWorkspacePipelinesPipelines.model_rebuild()
GetWorkspacePipelinesPipelinesItems.model_rebuild()



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import PipelineError


class StopPipeline(BaseModel):
    stop_pipeline: "StopPipelineStopPipeline" = Field(alias="stopPipeline")


class StopPipelineStopPipeline(BaseModel):
    success: bool
    errors: List[PipelineError]


StopPipeline.model_rebuild()



"""Datasets package.

See https://github.com/BLSQ/openhexa/wiki/User-manual#datasets and
https://github.com/BLSQ/openhexa/wiki/Using-the-OpenHEXA-SDK#working-with-datasets for more information about OpenHEXA
dataset.
"""

from .dataset import Dataset, DatasetFile

__all__ = ["Dataset", "DatasetFile"]



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import PipelineRunStatus, PipelineType


class Pipelines(BaseModel):
    pipelines: "PipelinesPipelines"


class PipelinesPipelines(BaseModel):
    total_pages: int = Field(alias="totalPages")
    items: List["PipelinesPipelinesItems"]


class PipelinesPipelinesItems(BaseModel):
    id: Any
    code: str
    name: Optional[str]
    type: PipelineType
    current_version: Optional["PipelinesPipelinesItemsCurrentVersion"] = Field(
        alias="currentVersion"
    )
    last_runs: "PipelinesPipelinesItemsLastRuns" = Field(alias="lastRuns")


class PipelinesPipelinesItemsCurrentVersion(BaseModel):
    id: Any
    name: Optional[str]
    version_number: int = Field(alias="versionNumber")


class PipelinesPipelinesItemsLastRuns(BaseModel):
    items: List["PipelinesPipelinesItemsLastRunsItems"]


class PipelinesPipelinesItemsLastRunsItems(BaseModel):
    id: Any
    status: PipelineRunStatus
    execution_date: Optional[Any] = Field(alias="executionDate")


Pipelines.model_rebuild()
PipelinesPipelines.model_rebuild()
PipelinesPipelinesItems.model_rebuild()
PipelinesPipelinesItemsLastRuns.model_rebuild()



"""API interactions test module."""

import base64
import io
import tempfile
from pathlib import Path
from unittest import mock
from zipfile import ZipFile

from openhexa.cli.api import create_pipeline_structure, upload_pipeline


def test_create_pipeline_structure(settings):
    """Test create_pipeline_structure function."""
    with tempfile.TemporaryDirectory() as temp_dir:
        pipeline_dir = create_pipeline_structure(
            "My pipeline",
            Path(temp_dir),
            workspace=settings.current_workspace,
            workflow_mode="manual",
        )

        assert (pipeline_dir / "pipeline.py").exists()
        assert (pipeline_dir / "workspace").exists()
        assert (pipeline_dir / "workspace.yaml").exists()

        assert (
            "name: Push pipeline on OpenHEXA"
            in (pipeline_dir / ".github" / "workflows" / "push-pipeline.yml").read_text()
        )
        assert "workflow_dispatch" in (pipeline_dir / ".github" / "workflows" / "push-pipeline.yml").read_text()

        assert "def my_pipeline(" in (pipeline_dir / "pipeline.py").read_text()


def test_upload_pipeline_success(settings):
    """Test upload API call."""
    with tempfile.TemporaryDirectory() as temp_dir:
        pipeline_dir = create_pipeline_structure(
            "my_pipeline",
            Path(temp_dir),
            workspace="workspace-slug",
        )

        # create a sample file inside workspace dir
        assert (Path(temp_dir) / "my_pipeline").exists()
        assert (pipeline_dir / "pipeline.py").exists()
        assert (pipeline_dir / "workspace").exists(), "Workspace directory not created"

        # Add files to the workspace directory
        with open(pipeline_dir / "workspace" / "dummy.txt", "w") as test_file:
            test_file.write("Test upload")

        # Add markdown file next to the pipeline.py
        with open(pipeline_dir / "readme.md", "w") as test_file:
            test_file.write("# README")

        with mock.patch("openhexa.cli.api.graphql") as mocked_graphql_client:
            mocked_graphql_client.return_value = {
                "uploadPipeline": {"pipelineVersion": {"name": "1"}, "success": True, "errors": []}
            }
            upload_pipeline(
                "target_pipeline_code", pipeline_dir, "version-name", "My description", "https://github.com/"
            )
            args_input = mocked_graphql_client.call_args[0][1]["input"]
            assert args_input["workspaceSlug"] == "workspace-slug"
            assert args_input["timeout"] is None
            assert args_input["parameters"] == []
            assert args_input["name"] == "version-name"
            assert args_input["description"] == "My description"
            assert args_input["externalLink"] == "https://github.com/"

            # Check if the zipfile is correctly created
            with ZipFile(io.BytesIO(base64.b64decode(args_input["zipfile"]))) as zip_file:
                assert "readme.md" in zip_file.namelist()
                assert "pipeline.py" in zip_file.namelist()
                assert len(zip_file.namelist()) == 2



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List, Literal, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import ConnectionType


class GetConnection(BaseModel):
    connection_by_slug: Optional["GetConnectionConnectionBySlug"] = Field(
        alias="connectionBySlug"
    )


class GetConnectionConnectionBySlug(BaseModel):
    typename__: Literal[
        "Connection",
        "CustomConnection",
        "DHIS2Connection",
        "GCSConnection",
        "IASOConnection",
        "PostgreSQLConnection",
        "S3Connection",
    ] = Field(alias="__typename")
    type: ConnectionType
    fields: List["GetConnectionConnectionBySlugFields"]


class GetConnectionConnectionBySlugFields(BaseModel):
    code: str
    value: Optional[str]


GetConnection.model_rebuild()
GetConnectionConnectionBySlug.model_rebuild()



"""Utilities used by containerized pipeline runners to import and download pipelines."""

import ast
import base64
import importlib
import io
import os
import sys
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any
from zipfile import ZipFile

import requests

from openhexa.sdk.pipelines.exceptions import InvalidParameterError, PipelineNotFound
from openhexa.sdk.pipelines.parameter import (
    TYPES_BY_PYTHON_TYPE,
    DHIS2Widget,
    IASOWidget,
    Parameter,
    validate_parameters,
)

from .pipeline import Pipeline


@dataclass
class Argument:
    """Argument of a decorator."""

    name: str  # Use str instead of string
    types: list[type] = field(default_factory=list)
    default_value: Any = None


def import_pipeline(pipeline_dir_path: str) -> Pipeline:
    """Import pipeline code within provided path using importlib.

    Args:
        pipeline_dir_path: Path to the directory containing the pipeline code

    Returns
    -------
        The imported Pipeline object

    Raises
    ------
        ImportError: If the pipeline module cannot be imported
        ValueError: If no Pipeline object is found in the module
    """
    pipeline_dir = os.path.abspath(pipeline_dir_path)
    sys.path.append(pipeline_dir)
    try:
        pipeline_package = importlib.import_module("pipeline")

        # Find the first Pipeline object in the module
        for _, obj in pipeline_package.__dict__.items():
            if isinstance(obj, Pipeline):
                return obj

        raise ValueError("No Pipeline object found in the imported module")
    except ImportError as e:
        raise ImportError(f"Failed to import pipeline module: {e}")


def download_pipeline(url: str, token: str, run_id: str, target_dir: str) -> None:
    """Download pipeline code and unzip it into the target directory.

    Args:
        url: The base URL for the API
        token: Authentication token
        run_id: ID of the pipeline run
        target_dir: Directory where the pipeline code will be extracted

    Raises
    ------
        requests.RequestException: If the API request fails
        ValueError: If the response data is invalid
    """
    query = """
    query PipelineDownload($id: UUID!) {
      pipelineRun(id: $id) {
        id
        version {
          versionNumber
        }
        code
      }
    }
    """

    try:
        response = requests.post(
            f"{url}/graphql/",
            headers={"Authorization": f"Bearer {token}"},
            json={"query": query, "variables": {"id": run_id}},
            timeout=30,
        )
        response.raise_for_status()

        data = response.json()
        if not data.get("data", {}).get("pipelineRun", {}).get("code"):
            raise ValueError("Invalid response: missing pipeline code")

        zipfile_data = base64.b64decode(data["data"]["pipelineRun"]["code"].encode("ascii"))

        # Use the original directory as a context to return to
        original_dir = os.getcwd()
        try:
            os.chdir(target_dir)
            with ZipFile(io.BytesIO(zipfile_data)) as zf:
                zf.extractall()
        finally:
            os.chdir(original_dir)
    except requests.RequestException as e:
        raise requests.RequestException(f"Failed to download pipeline: {e}")


def _get_decorators_by_name(node: ast.AST, name: str) -> list[ast.Call]:
    """Get all decorators with the specified name from an AST node.

    Args:
        node: The AST node to inspect
        name: The name of the decorator to find

    Returns
    -------
        List of matching decorator Call nodes
    """
    if not hasattr(node, "decorator_list"):
        return []

    return [
        dec
        for dec in node.decorator_list
        if isinstance(dec, ast.Call) and hasattr(dec.func, "id") and dec.func.id == name
    ]


def _get_decorator_arg_value(decorator: ast.Call, arg: Argument, index: int) -> tuple[Any, bool]:
    """Return the value of the argument of the decorator.

    Args:
        decorator: The decorator Call node
        arg: The Argument definition to extract
        index: The position index for positional arguments

    Returns
    -------
        A tuple with the value of the argument and a boolean indicating if
        the argument is a keyword argument.

    Raises
    ------
        ValueError: If the argument type is unsupported
    """
    # First check for keyword arguments
    for keyword in decorator.keywords:
        if keyword.arg == arg.name:
            if type(keyword.value) not in arg.types:
                raise ValueError(
                    f"Unsupported argument type for {arg.name}: {type(keyword.value)}. Expected {arg.types}"
                )
            if isinstance(keyword.value, ast.Constant):
                return (keyword.value.value, True)
            elif isinstance(keyword.value, ast.Name):
                return (keyword.value.id, True)
            elif isinstance(keyword.value, ast.List):
                return ([el.value for el in keyword.value.elts], True)
            elif isinstance(keyword.value, ast.Attribute):
                if keyword.value.attr in DHIS2Widget.__members__:
                    return getattr(DHIS2Widget, keyword.value.attr), True
                elif keyword.value.attr in IASOWidget.__members__:
                    return getattr(IASOWidget, keyword.value.attr), True
                else:
                    raise ValueError(f"Unsupported widget: {keyword.value.attr}")

    # Then check for positional arguments
    try:
        return (decorator.args[index].value, False)
    except IndexError:
        return (arg.default_value, False)


def _get_decorator_spec(decorator: ast.Call, args: tuple[Argument, ...]) -> dict[str, dict[str, Any]]:
    """Build a specification of decorator arguments.

    Args:
        decorator: The decorator Call node
        args: Tuple of Argument definitions to extract

    Returns
    -------
        Dictionary mapping argument names to their values and keyword status
    """
    args_spec = {}
    for i, arg in enumerate(args):
        value, is_keyword = _get_decorator_arg_value(decorator, arg, i)
        args_spec[arg.name] = {"value": value, "is_keyword": is_keyword}
    return args_spec


def get_pipeline(pipeline_path: Path) -> Pipeline:
    """Return the pipeline with metadata and parameters from the pipeline code.

    Args:
        pipeline_path: Path to the pipeline directory

    Raises
    ------
        PipelineNotFound: If no function with openhexa.sdk pipeline decorator is found.
        InvalidParameterError: If the parameter type is invalid/unknown.
        ValueError: If the value of an argument is not a primitive type.

    Returns
    -------
        Pipeline: The pipeline object with parameters and metadata.
    """
    pipeline_file = Path(pipeline_path) / "pipeline.py"

    try:
        with open(pipeline_file) as f:
            tree = ast.parse(f.read())
    except (FileNotFoundError, PermissionError) as e:
        raise PipelineNotFound(f"Could not read pipeline file: {e}")

    for node in ast.walk(tree):
        if not isinstance(node, ast.FunctionDef):
            continue

        pipeline_decorators = _get_decorators_by_name(node, "pipeline")
        if not pipeline_decorators:
            continue

        pipeline_decorator = pipeline_decorators[0]
        pipeline_args = _get_decorator_spec(
            pipeline_decorator,
            (
                Argument("code", [ast.Constant]),
                Argument("name", [ast.Constant]),
                Argument("timeout", [ast.Constant]),
            ),
        )

        # Extract code and name values for validation
        code_arg = pipeline_args.get("code", {"value": None, "is_keyword": False})
        name_arg = pipeline_args.get("name", {"value": None, "is_keyword": False})

        # Handle deprecated 'code' argument
        if code_arg["value"] is not None:
            if code_arg["is_keyword"]:
                print(
                    "\n\033[93m",
                    f"The 'code' argument is deprecated and should not be used as a keyword. "
                    f"Replace 'code=\"{code_arg['value']}\"' by 'name=\"{code_arg['value']}\"'\033[0m",
                    "\n",
                    flush=True,
                )

            if name_arg["value"] is not None:
                print(
                    "\n\033[93m",
                    f"Providing both 'code' and 'name' is deprecated. "
                    f"Please remove 'code' and only use 'name' when decorating the pipeline: "
                    f'@pipeline(name="{name_arg["value"]}")\033[0m',
                    "\n",
                    flush=True,
                )

        # Determine the pipeline name (prefer 'name', fall back to 'code')
        pipeline_name = name_arg["value"] if name_arg["value"] is not None else code_arg["value"]

        # Extract timeout
        timeout = pipeline_args.get("timeout", {"value": None})["value"]

        # Process parameters
        pipeline_parameters = []
        for parameter_decorator in _get_decorators_by_name(node, "parameter"):
            parameter_args = _get_decorator_spec(
                parameter_decorator,
                (
                    Argument("code", [ast.Constant]),
                    Argument("type", [ast.Name]),
                    Argument("name", [ast.Constant]),
                    Argument("choices", [ast.List]),
                    Argument("help", [ast.Constant]),
                    Argument("default", [ast.Constant, ast.List]),
                    Argument("widget", [ast.Attribute]),
                    Argument("connection", [ast.Constant]),
                    Argument("required", [ast.Constant], default_value=True),
                    Argument("multiple", [ast.Constant], default_value=False),
                ),
            )

            try:
                arg_type_name = parameter_args.pop("type")["value"]
                if arg_type_name not in TYPES_BY_PYTHON_TYPE:
                    raise InvalidParameterError(f"Unsupported parameter type: {arg_type_name}")

                type_class = TYPES_BY_PYTHON_TYPE[arg_type_name]()

                # Convert args spec to parameter kwargs
                param_kwargs = {k: v["value"] for k, v in parameter_args.items()}

                parameter = Parameter(type=type_class.expected_type, **param_kwargs)
                pipeline_parameters.append(parameter)

            except KeyError as e:
                raise InvalidParameterError(f"Missing required parameter attribute: {e}")

        validate_parameters(pipeline_parameters)

        # Create and return the pipeline
        return Pipeline(
            parameters=pipeline_parameters,
            function=None,
            name=pipeline_name,
            timeout=timeout,
        )

    # If we get here, no pipeline was found
    raise PipelineNotFound("No function with openhexa.sdk pipeline decorator found.")



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class Workspace(BaseModel):
    workspace: Optional["WorkspaceWorkspace"]


class WorkspaceWorkspace(BaseModel):
    slug: str
    name: str
    description: Optional[str]
    docker_image: Optional[str] = Field(alias="dockerImage")
    configuration: Optional[Any]
    countries: List["WorkspaceWorkspaceCountries"]
    permissions: "WorkspaceWorkspacePermissions"


class WorkspaceWorkspaceCountries(BaseModel):
    code: str
    flag: str
    name: str
    alpha_3: str = Field(alias="alpha3")


class WorkspaceWorkspacePermissions(BaseModel):
    delete: bool
    update: bool
    manage_members: bool = Field(alias="manageMembers")


Workspace.model_rebuild()
WorkspaceWorkspace.model_rebuild()



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class Dataset(BaseModel):
    dataset: Optional["DatasetDataset"]


class DatasetDataset(BaseModel):
    id: str
    slug: str
    name: str
    description: Optional[str]
    created_at: Any = Field(alias="createdAt")
    updated_at: Any = Field(alias="updatedAt")
    created_by: Optional["DatasetDatasetCreatedBy"] = Field(alias="createdBy")
    permissions: "DatasetDatasetPermissions"
    versions: "DatasetDatasetVersions"
    workspace: Optional["DatasetDatasetWorkspace"]


class DatasetDatasetCreatedBy(BaseModel):
    id: Any
    display_name: str = Field(alias="displayName")
    email: str


class DatasetDatasetPermissions(BaseModel):
    update: bool
    delete: bool
    create_version: bool = Field(alias="createVersion")


class DatasetDatasetVersions(BaseModel):
    items: List["DatasetDatasetVersionsItems"]


class DatasetDatasetVersionsItems(BaseModel):
    id: str
    name: str
    changelog: Optional[str]
    created_at: Any = Field(alias="createdAt")
    created_by: Optional["DatasetDatasetVersionsItemsCreatedBy"] = Field(
        alias="createdBy"
    )


class DatasetDatasetVersionsItemsCreatedBy(BaseModel):
    id: Any
    display_name: str = Field(alias="displayName")


class DatasetDatasetWorkspace(BaseModel):
    slug: str
    name: str


Dataset.model_rebuild()
DatasetDataset.model_rebuild()
DatasetDatasetVersions.model_rebuild()
DatasetDatasetVersionsItems.model_rebuild()



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import CreateWorkspaceError


class CreateWorkspace(BaseModel):
    create_workspace: "CreateWorkspaceCreateWorkspace" = Field(alias="createWorkspace")


class CreateWorkspaceCreateWorkspace(BaseModel):
    success: bool
    workspace: Optional["CreateWorkspaceCreateWorkspaceWorkspace"]
    errors: List[CreateWorkspaceError]


class CreateWorkspaceCreateWorkspaceWorkspace(BaseModel):
    slug: str
    name: str
    description: Optional[str]
    countries: List["CreateWorkspaceCreateWorkspaceWorkspaceCountries"]


class CreateWorkspaceCreateWorkspaceWorkspaceCountries(BaseModel):
    code: str
    alpha_3: str = Field(alias="alpha3")
    name: str


CreateWorkspace.model_rebuild()
CreateWorkspaceCreateWorkspace.model_rebuild()
CreateWorkspaceCreateWorkspaceWorkspace.model_rebuild()



"""OpenHexaClient implementation for GraphQL API interaction."""

import logging
from importlib.metadata import version

from openhexa.graphql.graphql_client import Client


class BaseOpenHexaClient(Client):
    """OpenHexaClient is a class that provides methods to interact with the OpenHexa GraphQL API."""

    def __init__(self, url: str, token: str):
        """Initialize the OpenHexaClient with the OpenHexa API URL and headers.

        Args:
            url: GraphQL API URL.
            token: Authentication token.
        """
        self.token = token
        super().__init__(
            url=url,
            headers={
                "User-Agent": f"openhexa-sdk/{version('openhexa.sdk')}",
                "Authorization": f"Bearer {self.token}",
            },
        )
        logging.getLogger("httpx").setLevel(
            logging.WARNING
        )  # HTTPX logs queries by default, we disable them here with WARNING level



"""Collection of tests related to environment variables handling."""

import os
from unittest.mock import Mock, patch

from openhexa.sdk.pipelines.pipeline import Pipeline


@patch.dict(os.environ, {"WORKSPACE_FILES_PATH": "/workspace/path"})
def test_workspace_local_files_path(workspace):
    """Test workspace files_path property with explicit environment variable."""
    assert workspace.files_path == "/workspace/path"


@patch.dict(os.environ, {"HEXA_ENVIRONMENT": "LOCAL_PIPELINE"})
def test_workspace_pipeline_files_path(workspace):
    """Test workspace files_path property in local mode."""
    from openhexa.sdk.workspaces import workspace

    assert workspace.files_path == "/home/hexa/workspace"


@patch.dict(
    os.environ,
    {"HEXA_ENVIRONMENT": "CLOUD_PIPELINE", "HEXA_SERVER_URL": "https://test.openhexa.org"},
)
def test_connected(current_run):
    """Test run _connected property in CLOUD_PIPELINE mode."""
    assert current_run._connected is True


@patch.dict(
    os.environ,
    {
        "HEXA_ENVIRONMENT": "CLOUD_PIPELINE",
    },
)
def test_not_connected_missing_url(current_run):
    """Test run _connected property in CLOUD_PIPELINE mode (missing HEXA_SERVER_URL)."""
    assert current_run._connected is False


@patch.dict(
    os.environ,
    {
        "HEXA_ENVIRONMENT": "LOCAL_PIPELINE",
    },
)
def test_not_connected(current_run):
    """Test run _connected property in LOCAL_PIPELINE mode."""
    assert current_run._connected is False


@patch.dict(
    os.environ,
    {
        "HEXA_ENVIRONMENT": "LOCAL_PIPELINE",
    },
)
def test_pipeline_local():
    """Test pipeline _connected property in LOCAL_PIPELINE mode."""
    pipeline_func = Mock()

    pipeline = Pipeline("code", "pipeline", pipeline_func, [])
    assert pipeline._connected is False


@patch.dict(
    os.environ,
    {
        "HEXA_ENVIRONMENT": "CLOUD_PIPELINE",
        "HEXA_SERVER_URL": "https://test.openhexa.org",
    },
)
def test_pipeline_pipeline():
    """Test pipeline _connected property in CLOUD_PIPELINE mode."""
    pipeline_func = Mock()

    pipeline = Pipeline("code", "pipeline", pipeline_func, [])
    assert pipeline._connected is True



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel


class GetCountries(BaseModel):
    workspace: Optional["GetCountriesWorkspace"]


class GetCountriesWorkspace(BaseModel):
    countries: List["GetCountriesWorkspaceCountries"]


class GetCountriesWorkspaceCountries(BaseModel):
    code: str
    name: str
    flag: str
    alpha_3: str = Field(alias="alpha3")


GetCountries.model_rebuild()
GetCountriesWorkspace.model_rebuild()



"""Pipeline parameters classes and functions.

See https://github.com/BLSQ/openhexa/wiki/Writing-OpenHEXA-pipelines#pipeline-parameters for more information.
"""

import typing
from enum import StrEnum

from openhexa.sdk.datasets import Dataset
from openhexa.sdk.pipelines.exceptions import InvalidParameterError, ParameterValueError
from openhexa.sdk.pipelines.utils import validate_pipeline_parameter_code
from openhexa.sdk.workspaces import workspace
from openhexa.sdk.workspaces.connection import (
    Connection,
    CustomConnection,
    DHIS2Connection,
    GCSConnection,
    IASOConnection,
    PostgreSQLConnection,
    S3Connection,
)
from openhexa.sdk.workspaces.current_workspace import ConnectionDoesNotExist


class ParameterType:
    """Base class for parameter types. Those parameter types are used when using the @parameter decorator."""

    def spec_type(self) -> str:
        """Return a type string for the specs that are sent to the backend."""
        raise NotImplementedError

    @property
    def expected_type(self) -> type:
        """Returns the python type expected for values."""
        raise NotImplementedError

    @property
    def accepts_choices(self) -> bool:
        """Return True only if the parameter type supports the "choices" optional argument."""
        return True

    @property
    def accepts_multiple(self) -> bool:
        """Return True only if the parameter type supports multiple values."""
        return True

    @staticmethod
    def normalize(value: typing.Any) -> typing.Any:
        """If appropriate, subclasses can override this method to normalize empty values to None.

        This can be used to handle empty values and normalize them to None, or to perform type conversions, allowing us
        to allow multiple input types but still normalize everything to a single type.
        """
        return value

    def validate(self, value: typing.Any | None) -> typing.Any | None:
        """Validate the provided value for this type."""
        if not isinstance(value, self.expected_type):
            raise ParameterValueError(
                f"Invalid type for value {value} (expected {self.expected_type}, got {type(value)})"
            )
        return value

    def validate_default(self, value: typing.Any | None):
        """Validate the default value configured for this type."""
        self.validate(value)

    def __str__(self) -> str:
        """Cast parameter as string."""
        return str(self.expected_type)


class StringType(ParameterType):
    """Type class for string parameters."""

    @property
    def spec_type(self) -> str:
        """Return a type string for the specs that are sent to the backend."""
        return "str"

    @property
    def expected_type(self) -> type:
        """Returns the python type expected for values."""
        return str

    @staticmethod
    def normalize(value: typing.Any) -> str | None:
        """Strip leading and trailing whitespaces and convert empty strings to None."""
        if isinstance(value, str):
            normalized_value = value.strip()
        else:
            normalized_value = value

        if normalized_value == "":
            return None

        return normalized_value

    def validate_default(self, value: typing.Any | None):
        """Validate the default value configured for this type."""
        if value == "":
            raise ParameterValueError("Empty values are not accepted.")

        super().validate_default(value)


class Boolean(ParameterType):
    """Type class for boolean parameters."""

    @property
    def spec_type(self) -> str:
        """Return a type string for the specs that are sent to the backend."""
        return "bool"

    @property
    def expected_type(self) -> type:
        """Returns the python type expected for values."""
        return bool

    @property
    def accepts_choices(self) -> bool:
        """Return a type string for the specs that are sent to the backend."""
        return False

    @property
    def accepts_multiple(self) -> bool:
        """Return a type string for the specs that are sent to the backend."""
        return False


class Integer(ParameterType):
    """Type class for integer parameters."""

    @property
    def spec_type(self) -> str:
        """Return a type string for the specs that are sent to the backend."""
        return "int"

    @property
    def expected_type(self) -> type:
        """Returns the python type expected for values."""
        return int


class Float(ParameterType):
    """Type class for float parameters."""

    @property
    def spec_type(self) -> str:
        """Return a type string for the specs that are sent to the backend."""
        return "float"

    @property
    def expected_type(self) -> type:
        """Returns the python type expected for values."""
        return float

    @staticmethod
    def normalize(value: typing.Any) -> typing.Any:
        """Normalize int values to float values if appropriate."""
        if isinstance(value, int):
            return float(value)

        return value


class ConnectionParameterType(ParameterType):
    """Abstract base class for connection parameter type classes."""

    @property
    def accepts_choices(self) -> bool:
        """Return True only if the parameter type supports the "choice values."""
        return False

    @property
    def accepts_multiple(self) -> bool:
        """Return True only if the parameter type supports multiple values."""
        return False

    def validate_default(self, value: typing.Any | None):
        """Validate the default value configured for this type."""
        if value is None:
            return

        if not isinstance(value, str):
            raise InvalidParameterError("Default value for connection parameter type should be string.")
        elif value == "":
            raise ParameterValueError("Empty values are not accepted.")

    def validate(self, value: typing.Any | None) -> Connection:
        """Validate the provided value for this type."""
        if not isinstance(value, str):
            raise ParameterValueError(f"Invalid type for value {value} (expected {str}, got {type(value)})")

        try:
            return self.to_connection(value)
        except ConnectionDoesNotExist as e:
            raise ParameterValueError(str(e))

    def to_connection(self, value: str) -> Connection:
        """Build a connection instance from the provided value (which should be a connection identifier)."""
        raise NotImplementedError


class PostgreSQLConnectionType(ConnectionParameterType):
    """Type class for PostgreSQL connections."""

    @property
    def spec_type(self) -> str:
        """Return a type string for the specs that are sent to the backend."""
        return "postgresql"

    @property
    def expected_type(self) -> type:
        """Returns the python type expected for values."""
        return PostgreSQLConnection

    def to_connection(self, value: str) -> PostgreSQLConnection:
        """Build a PostgreSQL connection instance from the provided value (which should be a connection identifier)."""
        return workspace.postgresql_connection(value)


class S3ConnectionType(ConnectionParameterType):
    """Type class for S3 connections."""

    @property
    def spec_type(self) -> str:
        """Return a type string for the specs that are sent to the backend."""
        return "s3"

    @property
    def expected_type(self) -> type:
        """Returns the python type expected for values."""
        return S3Connection

    def to_connection(self, value: str) -> S3Connection:
        """Build a S3 connection instance from the provided value (which should be a connection identifier)."""
        return workspace.s3_connection(value)


class GCSConnectionType(ConnectionParameterType):
    """Type class for GCS connections."""

    @property
    def spec_type(self) -> str:
        """Return a type string for the specs that are sent to the backend."""
        return "gcs"

    @property
    def expected_type(self) -> type:
        """Returns the python type expected for values."""
        return GCSConnection

    def to_connection(self, value: str) -> GCSConnection:
        """Build a GCS connection instance from the provided value (which should be a connection identifier)."""
        return workspace.gcs_connection(value)


class DHIS2ConnectionType(ConnectionParameterType):
    """Type class for DHIS2 connections."""

    @property
    def spec_type(self) -> str:
        """Return a type string for the specs that are sent to the backend."""
        return "dhis2"

    @property
    def expected_type(self) -> type:
        """Returns the python type expected for values."""
        return DHIS2Connection

    def to_connection(self, value: str) -> DHIS2Connection:
        """Build a DHIS2 connection instance from the provided value (which should be a connection identifier)."""
        return workspace.dhis2_connection(value)


class IASOConnectionType(ConnectionParameterType):
    """Type class for IASO connections."""

    @property
    def spec_type(self) -> str:
        """Return a type string for the specs that are sent to the backend."""
        return "iaso"

    @property
    def expected_type(self) -> type:
        """Returns the python type expected for values."""
        return IASOConnection

    def to_connection(self, value: str) -> IASOConnection:
        """Build a IASO connection instance from the provided value (which should be a connection identifier)."""
        return workspace.iaso_connection(value)


class CustomConnectionType(ConnectionParameterType):
    """Type class for custom connections."""

    @property
    def spec_type(self) -> str:
        """Return a type string for the specs that are sent to the backend."""
        return "custom"

    @property
    def expected_type(self) -> type:
        """Returns the python type expected for values."""
        return CustomConnection

    def to_connection(self, value: str) -> CustomConnection:
        """Build a custom connection instance from the provided value (which should be a connection identifier)."""
        return workspace.custom_connection(value)


class DatasetType(ParameterType):
    """Type class for dataset parameter."""

    @property
    def spec_type(self) -> str:
        """Return a type string for the specs that are sent to the backend."""
        return "dataset"

    @property
    def expected_type(self) -> type:
        """Returns the python type expected for values."""
        return Dataset

    def validate_default(self, value: typing.Any | None):
        """Validate the default value configured for this type."""
        if value is None:
            return

        if not isinstance(value, str):
            raise InvalidParameterError("Default value for dataset parameter type should be string.")
        elif value == "":
            raise ParameterValueError("Empty values are not accepted.")

    def validate(self, value: typing.Any | None) -> Dataset:
        """Validate the provided value for this type."""
        if not isinstance(value, str):
            raise ParameterValueError(f"Invalid type for value {value} (expected {str}, got {type(value)})")

        try:
            return workspace.get_dataset(value)
        except ValueError as e:
            raise ParameterValueError(str(e))


TYPES_BY_PYTHON_TYPE = {
    "str": StringType,
    "bool": Boolean,
    "int": Integer,
    "float": Float,
    "DHIS2Connection": DHIS2ConnectionType,
    "PostgreSQLConnection": PostgreSQLConnectionType,
    "IASOConnection": IASOConnectionType,
    "S3Connection": S3ConnectionType,
    "GCSConnection": GCSConnectionType,
    "CustomConnection": CustomConnectionType,
    "Dataset": DatasetType,
}


class IASOWidget(StrEnum):
    """Enum for IASO widgets."""

    IASO_FORMS = "IASO_FORMS"
    IASO_ORG_UNITS = "IASO_ORG_UNITS"
    IASO_PROJECTS = "IASO_PROJECTS"


class DHIS2Widget(StrEnum):
    """Enum for DHIS2 widgets."""

    ORG_UNITS = "DHIS2_ORG_UNITS"
    ORG_UNIT_GROUPS = "DHIS2_ORG_UNIT_GROUPS"
    ORG_UNIT_LEVELS = "DHIS2_ORG_UNIT_LEVELS"
    DATASETS = "DHIS2_DATASETS"
    DATA_ELEMENTS = "DHIS2_DATA_ELEMENTS"
    DATA_ELEMENT_GROUPS = "DHIS2_DATA_ELEMENT_GROUPS"
    INDICATORS = "DHIS2_INDICATORS"
    INDICATOR_GROUPS = "DHIS2_INDICATOR_GROUPS"


class Parameter:
    """Pipeline parameter class. Contains validation logic specs generation logic."""

    def __init__(
        self,
        code: str,
        *,
        type: type[
            str
            | int
            | bool
            | float
            | DHIS2Connection
            | IASOConnection
            | PostgreSQLConnection
            | GCSConnection
            | S3Connection
            | CustomConnection
            | Dataset
        ],
        name: str | None = None,
        choices: typing.Sequence | None = None,
        help: str | None = None,
        default: typing.Any | None = None,
        widget: DHIS2Widget | IASOWidget | None = None,
        connection: str | None = None,
        required: bool = True,
        multiple: bool = False,
    ):
        validate_pipeline_parameter_code(code)
        self.code = code

        try:
            self.type = TYPES_BY_PYTHON_TYPE[type.__name__]()
        except (KeyError, AttributeError):
            valid_parameter_types = [k for k in TYPES_BY_PYTHON_TYPE.keys()]
            raise InvalidParameterError(
                f"Invalid parameter type provided ({type}). "
                f"Valid parameter types are {', '.join(valid_parameter_types)}"
            )

        if choices is not None:
            if not self.type.accepts_choices:
                raise InvalidParameterError(f"Parameters of type {self.type} don't accept choices.")
            elif len(choices) == 0:
                raise InvalidParameterError("Choices, if provided, cannot be empty.")

            try:
                for choice in choices:
                    self.type.validate(choice)
            except ParameterValueError:
                raise InvalidParameterError(f"The provided choices are not valid for the {self.type} parameter type.")
        self.choices = choices

        self.name = name
        self.help = help
        self.required = required

        if multiple is True and not self.type.accepts_multiple:
            raise InvalidParameterError(f"Parameters of type {self.type} can't have multiple values.")
        self.multiple = multiple

        self.widget = widget
        self.connection = connection

        self._validate_default(default, multiple)
        self.default = default

    def validate(self, value: typing.Any) -> typing.Any:
        """Validate the provided value against the parameter, taking required / default options into account."""
        if self.multiple:
            return self._validate_multiple(value)
        else:
            return self._validate_single(value)

    def to_dict(self) -> dict[str, typing.Any]:
        """Return a dictionary representation of the Parameter instance."""
        return {
            "code": self.code,
            "type": self.type.spec_type,
            "name": self.name,
            "choices": self.choices,
            "help": self.help,
            "default": self.default,
            "widget": self.widget.value if self.widget else None,
            "connection": self.connection,
            "required": self.required,
            "multiple": self.multiple,
        }

    def _validate_single(self, value: typing.Any):
        # Normalize empty values to None and handles default
        normalized_value = self.type.normalize(value)
        if normalized_value is None and self.default is not None:
            normalized_value = self.default

        if normalized_value is None:
            if self.required:
                raise ParameterValueError(f"{self.code} is required")

            return None

        pre_validated = self.type.validate(normalized_value)
        if self.choices is not None and pre_validated not in self.choices:
            raise ParameterValueError(f"The provided value for {self.code} is not included in the provided choices.")

        return pre_validated

    def _validate_multiple(self, value: typing.Any):
        # Reject values that are not lists
        if value is not None and not isinstance(value, list):
            raise InvalidParameterError("If provided, value should be a list when parameter is multiple.")

        # Normalize empty values to an empty list
        if value is None:
            normalized_value = []
        else:
            normalized_value = [self.type.normalize(v) for v in value]
            normalized_value = list(filter(lambda v: v is not None, normalized_value))
        if len(normalized_value) == 0 and self.default is not None:
            normalized_value = self.default

        if len(normalized_value) == 0 and self.required:
            raise ParameterValueError(f"{self.code} is required")

        pre_validated = [self.type.validate(single_value) for single_value in normalized_value]
        if self.choices is not None and any(v not in self.choices for v in pre_validated):
            raise ParameterValueError(
                f"One of the provided values for {self.code} is not included in the provided choices."
            )

        return pre_validated

    def _validate_default(self, default: typing.Any, multiple: bool):
        if default is None:
            return

        try:
            if multiple:
                if not isinstance(default, list):
                    raise InvalidParameterError("Default values should be lists when using multiple=True")
                for default_value in default:
                    self.type.validate_default(default_value)
            else:
                self.type.validate_default(default)
        except ParameterValueError:
            raise InvalidParameterError(f"The default value for {self.code} is not valid.")

        if self.choices is not None:
            if isinstance(default, list):
                if not all(d in self.choices for d in default):
                    raise InvalidParameterError(
                        f"The default list of values for {self.code} is not included in the provided choices."
                    )
            elif default not in self.choices:
                raise InvalidParameterError(
                    f"The default value for {self.code} is not included in the provided choices."
                )


def validate_parameters(parameters: list[Parameter]):
    """Validate the provided connection parameters if they relate to existing connection parameter."""
    supported_connection_types = {DHIS2ConnectionType, IASOConnectionType}
    connection_parameters = {p.code for p in parameters if type(p.type) in supported_connection_types}

    for parameter in parameters:
        if parameter.connection and parameter.connection not in connection_parameters:
            raise InvalidParameterError(
                f"Connection field '{parameter.code}' references a non-existing connection parameter '{parameter.connection}'"
            )
        if (
            parameter.widget
            and (parameter.widget in DHIS2Widget or parameter.widget in IASOWidget)
            and not parameter.connection
        ):
            raise InvalidParameterError(
                f"Widgets require a connection parameter. Please provide a connection parameter for {parameter.code}. "
                f"Example: @parameter('my_connection', ...)"
                f"Example: @parameter('{parameter.code}', widget = ..., connection='my_connection')"
            )


def parameter(
    code: str,
    *,
    type: type[
        str
        | int
        | bool
        | float
        | DHIS2Connection
        | IASOConnection
        | PostgreSQLConnection
        | GCSConnection
        | S3Connection
        | CustomConnection
        | Dataset
    ],
    name: str | None = None,
    choices: typing.Sequence | None = None,
    help: str | None = None,
    widget: DHIS2Widget | IASOWidget | None = None,
    connection: str | None = None,
    default: typing.Any | None = None,
    required: bool = True,
    multiple: bool = False,
):
    """Decorate a pipeline function by attaching a parameter to it..

    This decorator must be used on a function decorated by the @pipeline decorator.

    Parameters
    ----------
    code : str
        The parameter identifier (must be unique for a given pipeline)
    type : {str, int, bool, float, DHIS2Connection, IASOConnection, PostgreSQLConnection, GCSConnection, S3Connection}
        The parameter Python type
    name : str, optional
        A name for the parameter (will be used instead of the code in the web interface)
    choices : list, optional
        An optional list or tuple of choices for the parameter (will be used to build a choice widget in the web
        interface)
    help : str, optional
        An optional help text to be displayed in the web interface
    widget : DHIS2Widget|IASOWidget, optional
        An optional widget type for the parameter (only used if the parameter type is DHIS2Connection, IASOConnection)
    connection : str, optional
        An optional connection parameter that will be used to link widget to the connection.
    default : any, optional
        An optional default value for the parameter (should be of the type defined by the type parameter)
    required : bool, default=True
        Whether the parameter is mandatory
    multiple : bool, default=True
        Whether this parameter should be provided multiple values (if True, the value must be provided as a list of
        values of the chosen type)

    Returns
    -------
    typing.Callable
        A decorator that returns the Pipeline with the parameter attached

    """

    def decorator(fun):
        return FunctionWithParameter(
            fun,
            Parameter(
                code,
                type=type,
                name=name,
                choices=choices,
                help=help,
                default=default,
                required=required,
                widget=widget,
                connection=connection,
                multiple=multiple,
            ),
        )

    return decorator


class FunctionWithParameter:
    """Wrapper class for pipeline functions decorated with the @parameter decorator."""

    def __init__(self, function, added_parameter: Parameter):
        self.function = function
        self.parameter = added_parameter

    def get_all_parameters(self) -> list[Parameter]:
        """Go through the decorators chain to find all pipeline parameters."""
        if isinstance(self.function, FunctionWithParameter):
            return [self.parameter, *self.function.get_all_parameters()]

        return [self.parameter]

    def __call__(self, *args, **kwargs):
        """Call the decorated pipeline function."""
        return self.function(*args, **kwargs)



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, List

from pydantic import Field

from .base_model import BaseModel


class GetUsers(BaseModel):
    users: List["GetUsersUsers"]


class GetUsersUsers(BaseModel):
    id: Any
    email: str
    display_name: str = Field(alias="displayName")
    avatar: "GetUsersUsersAvatar"


class GetUsersUsersAvatar(BaseModel):
    initials: str
    color: str


GetUsers.model_rebuild()
GetUsersUsers.model_rebuild()



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import PipelineError


class DeletePipeline(BaseModel):
    delete_pipeline: "DeletePipelineDeletePipeline" = Field(alias="deletePipeline")


class DeletePipelineDeletePipeline(BaseModel):
    success: bool
    errors: List[PipelineError]


DeletePipeline.model_rebuild()



"""SDK package."""

from .datasets import Dataset
from .pipelines import current_pipeline, current_run, parameter, pipeline
from .pipelines.parameter import DHIS2Widget, IASOWidget
from .utils import OpenHexaClient
from .workspaces import workspace
from .workspaces.connection import (
    CustomConnection,
    DHIS2Connection,
    GCSConnection,
    IASOConnection,
    PostgreSQLConnection,
    S3Connection,
)

__all__ = [
    "workspace",
    "pipeline",
    "current_pipeline",
    "parameter",
    "current_run",
    "DHIS2Connection",
    "DHIS2Widget",
    "IASOConnection",
    "IASOWidget",
    "PostgreSQLConnection",
    "GCSConnection",
    "S3Connection",
    "CustomConnection",
    "Dataset",
    "OpenHexaClient",
]



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import DeleteConnectionError


class DeleteConnection(BaseModel):
    delete_connection: "DeleteConnectionDeleteConnection" = Field(
        alias="deleteConnection"
    )


class DeleteConnectionDeleteConnection(BaseModel):
    success: bool
    errors: List[DeleteConnectionError]


DeleteConnection.model_rebuild()



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import PipelineTemplateError


class DeletePipelineTemplate(BaseModel):
    delete_pipeline_template: "DeletePipelineTemplateDeletePipelineTemplate" = Field(
        alias="deletePipelineTemplate"
    )


class DeletePipelineTemplateDeletePipelineTemplate(BaseModel):
    success: bool
    errors: List[PipelineTemplateError]


DeletePipelineTemplate.model_rebuild()



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import PipelineError


class CreatePipeline(BaseModel):
    create_pipeline: "CreatePipelineCreatePipeline" = Field(alias="createPipeline")


class CreatePipelineCreatePipeline(BaseModel):
    success: bool
    errors: List[PipelineError]
    pipeline: Optional["CreatePipelineCreatePipelinePipeline"]


class CreatePipelineCreatePipelinePipeline(BaseModel):
    code: str


CreatePipeline.model_rebuild()
CreatePipelineCreatePipeline.model_rebuild()



from unittest.mock import ANY, patch

from openhexa.sdk.pipelines.run import CurrentRun, LogLevel


@patch.object(CurrentRun, "_connected", True)
@patch("openhexa.sdk.pipelines.run.graphql")
def test_default_log_level(mock_graphql):
    current_run = CurrentRun()

    current_run.log_debug("This is a debug message")
    current_run.log_info("This is an info message")

    assert mock_graphql.call_count == 1
    mock_graphql.assert_any_call(ANY, {"input": {"priority": "INFO", "message": "This is an info message"}})


@patch.object(CurrentRun, "_connected", True)
@patch("openhexa.sdk.pipelines.run.graphql")
def test_filtering_log_messages_based_on_settings(mock_graphql, settings):
    settings.log_level = LogLevel.ERROR
    current_run = CurrentRun()

    current_run.log_warning("This is a warning message")
    current_run.log_error("This is an error message")
    current_run.log_critical("This is a critical message")

    assert mock_graphql.call_count == 2
    mock_graphql.assert_any_call(ANY, {"input": {"priority": "ERROR", "message": "This is an error message"}})
    mock_graphql.assert_any_call(ANY, {"input": {"priority": "CRITICAL", "message": "This is a critical message"}})



"""Custom HttpClient with retry mechanism."""

import requests
from requests import Session
from requests.adapters import HTTPAdapter
from urllib3.util import Retry


def create_requests_session(
    retries=3,
    backoff_factor=0.3,
    status_forcelist=(500, 502, 504),
) -> Session:
    """Return a Session object with retry capability."""
    session = requests.Session()
    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
    )
    adapter = HTTPAdapter(max_retries=retry)
    session.mount("http://", adapter)
    session.mount("https://", adapter)
    return session



"""Utilities for running local pipelines."""

import re
from pathlib import Path
from tempfile import mkdtemp

import yaml

from openhexa.sdk.pipelines.exceptions import InvalidParameterError
from openhexa.utils import stringcase


class LocalWorkspaceConfigError(Exception):
    """Raised whenever the local workspace config file does not exist or is invalid."""

    pass


def validate_pipeline_parameter_code(code: str):
    """Validate a pipeline parameter code."""
    if re.match("^[a-z_][a-z_0-9]+$", code) is None:
        raise InvalidParameterError(
            f"Invalid parameter code provided ({code}). Parameter must start with a letter or an underscore, "
            f"and can only contain lower case letters, numbers and underscores."
        )


def get_local_workspace_config(path: Path):
    """Get local workspace config for development/testing purposes.

    As of now, this function attempts to make the local run context looks like an online run context by
    filling environment variables.

    This is obviously brittle as it relies on setting the correct env variables keys, any changes upstream must
    be reflected here.
    """
    env_vars = {}

    # This will only work when running the pipeline using "python pipeline.py"
    # (We will have to find another approach for tests or running the pipeline using the CLI)
    local_workspace_config_path = path / Path("workspace.yaml")
    if not local_workspace_config_path.exists():
        raise FileNotFoundError(
            "To work with pipelines locally, you need a workspace.yaml file in the same directory as your pipeline file"
        )

    with open(local_workspace_config_path.resolve()) as local_workspace_config_file:
        local_workspace_config = yaml.safe_load(local_workspace_config_file)
        # Database config
        if "database" in local_workspace_config:
            try:
                env_vars["WORKSPACE_DATABASE_USERNAME"] = local_workspace_config["database"]["username"]
                password = local_workspace_config["database"].get("password")
                if password is not None:
                    env_vars["WORKSPACE_DATABASE_PASSWORD"] = password
                env_vars["WORKSPACE_DATABASE_HOST"] = local_workspace_config["database"]["host"]
                env_vars["WORKSPACE_DATABASE_PORT"] = str(local_workspace_config["database"]["port"])
                env_vars["WORKSPACE_DATABASE_DB_NAME"] = local_workspace_config["database"]["dbname"]
            except KeyError:
                exception_message = (
                    "Invalid local workspace database config. Please make sure you provide the following "
                    "keys: database.username, database.password, database.host, database.port, "
                    "database.dbname."
                )
                raise LocalWorkspaceConfigError(exception_message)

        # Files config
        if "files" in local_workspace_config:
            try:
                files_path = path / Path(local_workspace_config["files"]["path"])
                if files_path.exists() is False:
                    # Let's create the folder if it doesn't exist (only needed when running the pipeline using `python pipeline.py`)
                    files_path.mkdir(parents=True)
                env_vars["WORKSPACE_FILES_PATH"] = str(files_path.resolve())
            except KeyError:
                exception_message = (
                    "Invalid local workspace database config. Please make sure you provide the following "
                    "keys: username, password, host, port, name."
                )
                raise LocalWorkspaceConfigError(exception_message)

            # Create temp dir to simulate /home/hexa/tmp locally
            env_vars["WORKSPACE_TMP_PATH"] = mkdtemp()

        # Connections
        if "connections" in local_workspace_config:
            for slug, connection_config in local_workspace_config["connections"].items():
                slug = slug.lower()
                if "type" not in connection_config:
                    raise LocalWorkspaceConfigError("Each connection must have a type key.")

                # DHIS2 connection
                if connection_config["type"] == "dhis2":
                    try:
                        env_vars[f"{stringcase.constcase(slug)}"] = connection_config["type"]
                        env_vars[f"{stringcase.constcase(slug)}_URL"] = connection_config["url"]
                        env_vars[f"{stringcase.constcase(slug)}_USERNAME"] = connection_config["username"]
                        env_vars[f"{stringcase.constcase(slug)}_PASSWORD"] = connection_config["password"]
                    except KeyError:
                        exception_message = (
                            "Invalid local workspace dhis2 connection config. Please make sure you provide "
                            "the following keys: url, username, password."
                        )
                        raise LocalWorkspaceConfigError(exception_message)
                # PostgreSQL connection
                elif connection_config["type"] == "postgresql":
                    try:
                        env_vars[f"{stringcase.constcase(slug)}"] = connection_config["type"]
                        env_vars[f"{stringcase.constcase(slug)}_HOST"] = connection_config["host"]
                        env_vars[f"{stringcase.constcase(slug)}_PORT"] = str(connection_config["port"])
                        env_vars[f"{stringcase.constcase(slug)}_USERNAME"] = str(connection_config["username"])
                        env_vars[f"{stringcase.constcase(slug)}_PASSWORD"] = connection_config["password"]
                        env_vars[f"{stringcase.constcase(slug)}_DB_NAME"] = connection_config["database_name"]
                    except KeyError:
                        exception_message = (
                            "Invalid local workspace PostgreSQL connection config. Please make sure you provide "
                            "the following keys: url, username, password."
                        )
                        raise LocalWorkspaceConfigError(exception_message)
                # S3 connection
                elif connection_config["type"] == "s3":
                    try:
                        env_vars[f"{stringcase.constcase(slug)}"] = connection_config["type"]
                        env_vars[f"{stringcase.constcase(slug)}_ACCESS_KEY_SECRET"] = connection_config[
                            "secret_access_key"
                        ]
                        env_vars[f"{stringcase.constcase(slug)}_ACCESS_KEY_ID"] = str(
                            connection_config["access_key_id"]
                        )
                        env_vars[f"{stringcase.constcase(slug)}_BUCKET_NAME"] = str(connection_config["bucket_name"])
                    except KeyError:
                        exception_message = (
                            "Invalid local workspace S3 connection config. Please make sure you provide "
                            "the following keys: secret_key, access_key_id, bucket_name."
                        )
                        raise LocalWorkspaceConfigError(exception_message)
                # GCS connection
                elif connection_config["type"] == "gcs":
                    try:
                        env_vars[f"{stringcase.constcase(slug)}"] = connection_config["type"]
                        env_vars[f"{stringcase.constcase(slug)}_SERVICE_ACCOUNT_KEY"] = connection_config[
                            "service_account_key"
                        ]
                        env_vars[f"{stringcase.constcase(slug)}_BUCKET_NAME"] = str(connection_config["bucket_name"])
                    except KeyError:
                        exception_message = (
                            "Invalid local workspace GCS connection config. Please make sure you provide "
                            "the following keys: service_account_key, bucket_name."
                        )
                        raise LocalWorkspaceConfigError(exception_message)
                    # IASO connection
                elif connection_config["type"] == "iaso":
                    try:
                        env_vars[f"{stringcase.constcase(slug)}"] = connection_config["type"]
                        env_vars[f"{stringcase.constcase(slug)}_URL"] = connection_config["url"]
                        env_vars[f"{stringcase.constcase(slug)}_USERNAME"] = connection_config["username"]
                        env_vars[f"{stringcase.constcase(slug)}_PASSWORD"] = connection_config["password"]
                    except KeyError:
                        exception_message = (
                            "Invalid local workspace iaso connection config. Please make sure you provide "
                            "the following keys: url, username, password."
                        )
                        raise LocalWorkspaceConfigError(exception_message)
                # Custom connection
                else:
                    for key, value in connection_config.items():
                        if key != "type":
                            env_vars[stringcase.constcase(f"{slug}_{key.lower()}")] = str(value)
                    env_vars[f"{stringcase.constcase(slug)}"] = connection_config["type"]

        # Workspace docker image
        if "image" in local_workspace_config:
            env_vars["WORKSPACE_DOCKER_IMAGE"] = local_workspace_config["image"]

        # Additional environment variables
        if "env" in local_workspace_config:
            for key, value in local_workspace_config["env"].items():
                env_vars[key] = str(value)

    return env_vars



"""Main pipeline module containing the building blocks for OpenHEXA pipelines.

See https://github.com/BLSQ/openhexa/wiki/User-manual#using-pipelines and
https://github.com/BLSQ/openhexa/wiki/Writing-OpenHEXA-pipelines for more information about OpenHEXA pipelines.
"""

import argparse
import datetime
import json
import os
import sys
import time
import typing
from logging import getLogger
from pathlib import Path

import requests
from multiprocess import get_context  # NOQA

from openhexa.sdk.utils import Environment, get_environment

from .parameter import FunctionWithParameter, Parameter, ParameterValueError
from .task import PipelineWithTask, Task
from .utils import get_local_workspace_config

logger = getLogger(__name__)


class Pipeline:
    """OpenHEXA pipeline class.

    Pipeline are usually instantiated through the @pipeline decorator.

    Attributes
    ----------
    name : str
        A user-friendly name for the pipeline (will be displayed in the web interface).
    function: typing.Callable
        The actual pipeline function.
    parameters : typing.Sequence[Parameter]
        A list of Parameter instance corresponding to the pipeline parameters.
    timeout : int
        The timeout in seconds after which the pipeline will be killed.
    """

    def __init__(
        self,
        name: str,
        function: typing.Callable,
        parameters: typing.Sequence[Parameter],
        timeout: int = None,
    ):
        self.name = name
        self.function = function
        self.parameters = parameters
        self.timeout = timeout
        self.tasks = []

    def task(self, function) -> PipelineWithTask:
        """Task decorator.

        Examples
        --------
        >>> @pipeline("my-pipeline")
        ... def my_pipeline():
        ...     result_1 = task1()
        ...     task2(result_1)
        ...
        ... @my_pipeline.task
        ... def task_1() -> int:
        ...     return 42
        ...
        ... @my_pipeline.task
        ... def task_2(foo: int):
        ...     pass
        """
        return PipelineWithTask(function, self)

    def run(self, config: dict[str, typing.Any]):
        """Run the pipeline using the provided config.

        Parameters
        ----------
        config : typing.Dict[str, typing.Any]
            The parameter values to use for this pipeline run.
        """
        now = datetime.datetime.now(tz=datetime.UTC).replace(microsecond=0).isoformat()
        print(f'{now} Starting pipeline "{self.name}"')

        # Validate / default parameters
        validated_config = {}
        for parameter in self.parameters:
            value = config.pop(parameter.code, None)
            validated_value = parameter.validate(value)
            validated_config[parameter.code] = validated_value

        if len(config) > 0:
            raise ParameterValueError(f"The provided config contains invalid key(s): {', '.join(list(config.keys()))}")

        self.function(**validated_config)

        # managing variables
        result_list = []
        context = get_context("spawn")
        pool = context.Pool()  # FIXME: set max size of pool
        total = len(self.tasks)
        completed = 0

        while True:
            tasks = self._get_available_tasks()
            # filter already pooled task, even if they are not finished
            tasks = [t for t in tasks if not t.pooled]

            if len(tasks) == 0 and len(result_list) == 0:
                # nothing running, nothing to do
                break

            for task in tasks:
                now = datetime.datetime.now(tz=datetime.UTC).replace(microsecond=0).isoformat()
                print(f'{now} Started task "{task.compute.__name__}"')
                result = pool.apply_async(task.run)
                result_list.append((result, task))
                task.pooled = True

            while True:
                dag_step = False

                for result, task in result_list:
                    if not result.ready():
                        continue
                    now = datetime.datetime.now(tz=datetime.UTC).replace(microsecond=0).isoformat()

                    completed += 1
                    progress = int(completed / total * 100)
                    print(f'{now} Finished task "{task.compute.__name__}"')
                    self._update_progress(progress)

                    try:
                        task_com_result = result.get()
                        task.result = task_com_result.result
                        task.start_time = task_com_result.start_time
                        task.end_time = task_com_result.end_time
                        dag_step = True
                    except Exception as e:  # NOQA
                        raise PipelineRunError(f"Pipeline {self.name} failed: {e}")

                if dag_step:
                    # remove finished tasks
                    result_list = [(r, t) for r, t in result_list if t.end_time is None]
                    break
                else:
                    # busy loop
                    time.sleep(0.3)

        pool.close()
        pool.join()

        now = datetime.datetime.now(tz=datetime.UTC).replace(microsecond=0).isoformat()
        print(f'{now} Successfully completed pipeline "{self.name}"')

    def to_dict(self):
        """Return a dictionary representation of the pipeline."""
        return {
            "name": self.name,
            "parameters": [p.to_dict() for p in self.parameters],
            "timeout": self.timeout,
            "function": self.function.__dict__ if self.function else None,
            "tasks": [t.__dict__ for t in self.tasks],
        }

    def _get_available_tasks(self) -> list[Task]:
        return [task for task in self.tasks if task.is_ready()]

    def _update_progress(self, progress: int):
        if self._connected:
            token = os.environ["HEXA_TOKEN"]
            headers = {"Authorization": "Bearer %s" % token}
            query = """
                            mutation updatePipelineProgress ($input: UpdatePipelineProgressInput!) {
                                updatePipelineProgress(input: $input) { success errors }
                            }"""
            r = requests.post(
                f"{os.environ['HEXA_SERVER_URL']}/graphql/",
                headers=headers,
                json={
                    "query": query,
                    "variables": {"input": {"percent": progress}},
                },
            )
            r.raise_for_status()
        else:
            print(f"Progress update: {progress}%")

    @property
    def _connected(self) -> bool:
        env = get_environment()

        return env == Environment.CLOUD_PIPELINE and "HEXA_SERVER_URL" in os.environ

    def __call__(self, config: dict[str | typing.Any] | None = None):
        """Call the pipeline by running it, after having configured the environment.

        This method can be called with an explicit configuration. If no configuration is provided, it will parse the
        command-line arguments to build it.
        """
        # User can run their pipeline using `python pipeline.py`. It's considered as a standalone usage of the library.
        # Since we still support this use case for the moment, we'll try to load the workspace.yaml
        # at the path of the file
        if get_environment() == Environment.STANDALONE:
            os.environ.update(get_local_workspace_config(Path(sys.argv[0]).parent))

        if config is None:  # Called without arguments, in the pipeline file itself
            parser = argparse.ArgumentParser(exit_on_error=False)
            parser.add_argument("-c", "--config")
            parser.add_argument("-f", "--config-file")
            # We can't use parse_args, as it will call sys.exit() if there are unrecognized arguments
            args, argv = parser.parse_known_args()
            if argv or (args.config_file is not None and args.config is not None):
                raise ValueError(
                    f"Unrecognized arguments: {' '.join(argv)}. Running a pipeline requires a single "
                    "argument: either an inline JSON config with the --config/-c argument, or a JSON "
                    "config file with the --config-file/-f argument."
                )
            if args.config_file is not None:
                with open(args.config_file) as cf:
                    try:
                        config = json.load(cf)
                    except json.JSONDecodeError:
                        raise PipelineConfigError("The provided config is not valid JSON")

            elif args.config is not None:
                try:
                    config = json.loads(args.config)
                except json.JSONDecodeError:
                    raise PipelineConfigError("The provided config is not valid JSON")
            else:
                config = {}

        self.run(config)


def pipeline(
    code: str = None, name: str = None, timeout: int = None
) -> typing.Callable[[typing.Callable[..., typing.Any]], Pipeline]:
    """Decorate a Python function as an OpenHEXA pipeline.

    Parameters
    ----------
    code : str, optional
        Deprecated identifier for the pipeline. A unique identifier will be auto-generated.
    name : str
        A name for the pipeline (will be shown in the web interface).
    timeout : int, optional
        An optional timeout, in seconds, after which the pipeline run will be terminated (if not provided, a default
        timeout will be applied by the OpenHEXA backend)

    Returns
    -------
    typing.Callable
        A decorator that returns a Pipeline

    Examples
    --------
    >>> @pipeline("my-pipeline")
    ... def my_pipeline():
    ...     a_task()
    ...
    ... @my_pipeline.task
    ... def a_task() -> int:
    ...     return 42
    """
    name = name or code

    def decorator(fun):
        if isinstance(fun, FunctionWithParameter):
            parameters = fun.get_all_parameters()
        else:
            parameters = []

        return Pipeline(name, fun, parameters, timeout)

    return decorator


class PipelineConfigError(Exception):
    """Error raised whenver the config passed to the pipeline run method is invalid."""

    pass


class PipelineRunError(Exception):
    """Generic pipeline runtime error, raised whenever user code raises an exception."""

    pass



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import UpdateDatasetError


class UpdateDataset(BaseModel):
    update_dataset: "UpdateDatasetUpdateDataset" = Field(alias="updateDataset")


class UpdateDatasetUpdateDataset(BaseModel):
    success: bool
    errors: List[UpdateDatasetError]
    dataset: Optional["UpdateDatasetUpdateDatasetDataset"]


class UpdateDatasetUpdateDatasetDataset(BaseModel):
    slug: str
    name: str
    description: Optional[str]


UpdateDataset.model_rebuild()
UpdateDatasetUpdateDataset.model_rebuild()



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel
from .enums import CreateDatasetError


class CreateDataset(BaseModel):
    create_dataset: "CreateDatasetCreateDataset" = Field(alias="createDataset")


class CreateDatasetCreateDataset(BaseModel):
    success: bool
    errors: List[CreateDatasetError]
    dataset: Optional["CreateDatasetCreateDatasetDataset"]


class CreateDatasetCreateDatasetDataset(BaseModel):
    slug: str
    name: str


CreateDataset.model_rebuild()
CreateDatasetCreateDataset.model_rebuild()



"""Current Pipeline context module."""
import os

from openhexa.sdk.utils import Environment, get_environment


class PipelineConfigError(Exception):
    """Raised whenever the system cannot find an environment variable required to configure the current pipeline."""

    pass


class CurrentPipeline:
    """It is used to view manage the current pipeline state and operations."""

    @property
    def code(self) -> str:
        """The unique slug of the workspace.

        Slugs are used to identify the workspace.
        """
        try:
            return os.environ["HEXA_PIPELINE_CODE"]
        except KeyError:
            raise PipelineConfigError("The pipeline code is not available in this environment.")

    @property
    def name(self) -> str:
        """The name of the current pipeline."""
        try:
            return os.environ["HEXA_PIPELINE_NAME"]
        except KeyError:
            raise PipelineConfigError("The pipeline name is not available in this environment.")

    @property
    def type(self) -> str:
        """The type of the current pipeline."""
        try:
            return os.environ["HEXA_PIPELINE_TYPE"]
        except KeyError:
            raise PipelineConfigError("The pipeline type is not available in this environment.")


if get_environment() == Environment.CLOUD_JUPYTER:
    current_pipeline = None
else:
    current_pipeline = CurrentPipeline()



"""Simple module for a sample IO pipeline."""

import json

import pandas as pd
import requests
from sqlalchemy import create_engine

from openhexa.sdk import current_run, pipeline, workspace


@pipeline("Simple IO")
def simple_io():
    """Run a simple IO pipeline."""
    # Read and write from/to workspace files
    raw_files_data = load_files_data()
    transform_and_write_files_data(raw_files_data)

    # Read and write from/to workspace database
    raw_sql_data = load_data_from_postgresql()
    transform_and_write_sql_data(raw_sql_data)

    # Use connection
    load_dhis2_data()


@simple_io.task
def load_files_data():
    """Load data from workspace filesystem."""
    current_run.log_info("Loading files data...")

    return pd.read_csv(f"{workspace.files_path}/raw.csv")


@simple_io.task
def transform_and_write_files_data(raw_data: pd.DataFrame):
    """Simulate a transformation on the provided dataframe and write data to workspace filesystem."""
    current_run.log_info("Transforming files data...")

    transformed_data = raw_data.copy()
    transformed_data["foo"] = transformed_data["foo"].multiply(2)
    transformed_path = f"{workspace.files_path}/transformed.csv"
    transformed_data.to_csv(transformed_path, index=False)
    current_run.add_file_output(transformed_path)


@simple_io.task
def load_data_from_postgresql() -> pd.DataFrame:
    """Perform a simple SELECT query in the workspace database."""
    current_run.log_info("Loading Postgres data...")

    engine = create_engine(workspace.database_url)

    return pd.read_sql("SELECT * FROM foo", con=engine)


@simple_io.task
def transform_and_write_sql_data(raw_data: pd.DataFrame):
    """Simulate a transform operation on the provided data and load it in the workspace database."""
    current_run.log_info("Transforming postgres data...")

    engine = create_engine(workspace.database_url)
    transformed_data = raw_data.copy()
    transformed_data["bar"] = transformed_data["bar"].multiply(3)
    transformed_data.to_sql("baz", con=engine, index=False, if_exists="replace")
    current_run.add_database_output("baz")


@simple_io.task
def load_dhis2_data():
    """Load data from DHIS2."""
    current_run.log_info("Loading DHIS2 data...")

    connection = workspace.dhis2_connection("dhis2-play")
    base_url = f"{connection.url}/api"
    session = requests.Session()
    session.auth = (connection.username, connection.password)
    analytics_response = session.get(
        f"{base_url}/analytics",
        params={
            "aggregationType": "SUM",
            "dimension": [
                "dx:DE_GROUP-qfxEYY9xAl6",
                "pe:2021;2022",
                "ou:LEVEL-2",
            ],
        },
    )
    current_run.log_debug(f"Got {analytics_response.status_code} response from DHIS2")

    return json.loads(analytics_response.text)


if __name__ == "__main__":
    simple_io()



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import Any, Dict, List, Optional, Union

from .add_pipeline_recipient import (
    AddPipelineRecipient,
    AddPipelineRecipientAddPipelineRecipient,
)
from .add_webapp_to_favorites import (
    AddWebappToFavorites,
    AddWebappToFavoritesAddToFavorites,
)
from .archive_workspace import ArchiveWorkspace, ArchiveWorkspaceArchiveWorkspace
from .base_client import BaseClient
from .base_model import UNSET, UnsetType
from .create_connection import CreateConnection, CreateConnectionCreateConnection
from .create_dataset import CreateDataset, CreateDatasetCreateDataset
from .create_pipeline import CreatePipeline, CreatePipelineCreatePipeline
from .create_pipeline_from_template_version import (
    CreatePipelineFromTemplateVersion,
    CreatePipelineFromTemplateVersionCreatePipelineFromTemplateVersion,
)
from .create_pipeline_template_version import (
    CreatePipelineTemplateVersion,
    CreatePipelineTemplateVersionCreatePipelineTemplateVersion,
)
from .create_webapp import CreateWebapp, CreateWebappCreateWebapp
from .create_workspace import CreateWorkspace, CreateWorkspaceCreateWorkspace
from .dataset import Dataset, DatasetDataset
from .datasets import Datasets, DatasetsDatasets
from .delete_connection import DeleteConnection, DeleteConnectionDeleteConnection
from .delete_dataset import DeleteDataset, DeleteDatasetDeleteDataset
from .delete_pipeline import DeletePipeline, DeletePipelineDeletePipeline
from .delete_pipeline_template import (
    DeletePipelineTemplate,
    DeletePipelineTemplateDeletePipelineTemplate,
)
from .delete_pipeline_version import (
    DeletePipelineVersion,
    DeletePipelineVersionDeletePipelineVersion,
)
from .delete_webapp import DeleteWebapp, DeleteWebappDeleteWebapp
from .get_connection import GetConnection, GetConnectionConnectionBySlug
from .get_users import GetUsers, GetUsersUsers
from .input_types import (
    AddToFavoritesInput,
    ArchiveWorkspaceInput,
    CreateConnectionInput,
    CreateDatasetInput,
    CreatePipelineFromTemplateVersionInput,
    CreatePipelineInput,
    CreatePipelineRecipientInput,
    CreatePipelineTemplateVersionInput,
    CreateWebappInput,
    CreateWorkspaceInput,
    DeleteConnectionInput,
    DeleteDatasetInput,
    DeletePipelineInput,
    DeletePipelineTemplateInput,
    DeletePipelineVersionInput,
    DeleteWebappInput,
    InviteWorkspaceMemberInput,
    RemoveFromFavoritesInput,
    StopPipelineInput,
    UpdateConnectionInput,
    UpdateDatasetInput,
    UpdateWebappInput,
    UpdateWorkspaceInput,
    UpgradePipelineVersionFromTemplateInput,
    UploadPipelineInput,
)
from .invite_workspace_member import (
    InviteWorkspaceMember,
    InviteWorkspaceMemberInviteWorkspaceMember,
)
from .pipeline import Pipeline, PipelinePipelineByCode
from .pipelines import Pipelines, PipelinesPipelines
from .remove_webapp_from_favorites import (
    RemoveWebappFromFavorites,
    RemoveWebappFromFavoritesRemoveFromFavorites,
)
from .stop_pipeline import StopPipeline, StopPipelineStopPipeline
from .update_connection import UpdateConnection, UpdateConnectionUpdateConnection
from .update_dataset import UpdateDataset, UpdateDatasetUpdateDataset
from .update_webapp import UpdateWebapp, UpdateWebappUpdateWebapp
from .update_workspace import UpdateWorkspace, UpdateWorkspaceUpdateWorkspace
from .upgrade_pipeline_version_from_template import (
    UpgradePipelineVersionFromTemplate,
    UpgradePipelineVersionFromTemplateUpgradePipelineVersionFromTemplate,
)
from .upload_pipeline import UploadPipeline, UploadPipelineUploadPipeline
from .workspace import Workspace, WorkspaceWorkspace
from .workspaces import Workspaces, WorkspacesWorkspaces


def gql(q: str) -> str:
    return q


class Client(BaseClient):
    def pipelines(
        self,
        workspace_slug: str,
        name: Union[Optional[str], UnsetType] = UNSET,
        page: Union[Optional[int], UnsetType] = UNSET,
        per_page: Union[Optional[int], UnsetType] = UNSET,
        **kwargs: Any
    ) -> PipelinesPipelines:
        query = gql(
            """
            query Pipelines($workspaceSlug: String!, $name: String, $page: Int = 1, $perPage: Int = 10) {
              pipelines(
                workspaceSlug: $workspaceSlug
                name: $name
                page: $page
                perPage: $perPage
              ) {
                totalPages
                items {
                  id
                  code
                  name
                  type
                  currentVersion {
                    id
                    name
                    versionNumber
                  }
                  lastRuns: runs(orderBy: EXECUTION_DATE_DESC, page: 1, perPage: 1) {
                    items {
                      id
                      status
                      executionDate
                    }
                  }
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {
            "workspaceSlug": workspace_slug,
            "name": name,
            "page": page,
            "perPage": per_page,
        }
        response = self.execute(
            query=query, operation_name="Pipelines", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return Pipelines.model_validate(data).pipelines

    def pipeline(
        self, workspace_slug: str, pipeline_code: str, **kwargs: Any
    ) -> Optional[PipelinePipelineByCode]:
        query = gql(
            """
            query Pipeline($workspaceSlug: String!, $pipelineCode: String!) {
              pipelineByCode(workspaceSlug: $workspaceSlug, code: $pipelineCode) {
                id
                code
                name
                description
                type
                currentVersion {
                  id
                  name
                  versionNumber
                  user {
                    id
                    email
                    displayName
                    avatar {
                      initials
                      color
                    }
                  }
                  createdAt
                }
                runs(orderBy: EXECUTION_DATE_DESC, page: 1, perPage: 10) {
                  items {
                    id
                    status
                    executionDate
                    user {
                      id
                      displayName
                      email
                    }
                  }
                }
                webhookUrl
                webhookEnabled
                schedule
                notebookPath
                sourceTemplate {
                  id
                  code
                  name
                }
                hasNewTemplateVersions
                newTemplateVersions {
                  id
                  changelog
                  versionNumber
                  createdAt
                }
                recipients {
                  user {
                    id
                    displayName
                  }
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {
            "workspaceSlug": workspace_slug,
            "pipelineCode": pipeline_code,
        }
        response = self.execute(
            query=query, operation_name="Pipeline", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return Pipeline.model_validate(data).pipeline_by_code

    def create_pipeline(
        self, input: CreatePipelineInput, **kwargs: Any
    ) -> CreatePipelineCreatePipeline:
        query = gql(
            """
            mutation CreatePipeline($input: CreatePipelineInput!) {
              createPipeline(input: $input) {
                success
                errors
                pipeline {
                  code
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query, operation_name="CreatePipeline", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return CreatePipeline.model_validate(data).create_pipeline

    def upload_pipeline(
        self, input: UploadPipelineInput, **kwargs: Any
    ) -> UploadPipelineUploadPipeline:
        query = gql(
            """
            mutation uploadPipeline($input: UploadPipelineInput!) {
              uploadPipeline(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query, operation_name="uploadPipeline", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return UploadPipeline.model_validate(data).upload_pipeline

    def delete_pipeline(
        self, input: DeletePipelineInput, **kwargs: Any
    ) -> DeletePipelineDeletePipeline:
        query = gql(
            """
            mutation DeletePipeline($input: DeletePipelineInput!) {
              deletePipeline(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query, operation_name="DeletePipeline", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return DeletePipeline.model_validate(data).delete_pipeline

    def stop_pipeline(
        self, input: StopPipelineInput, **kwargs: Any
    ) -> StopPipelineStopPipeline:
        query = gql(
            """
            mutation StopPipeline($input: StopPipelineInput!) {
              stopPipeline(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query, operation_name="StopPipeline", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return StopPipeline.model_validate(data).stop_pipeline

    def add_pipeline_recipient(
        self, input: CreatePipelineRecipientInput, **kwargs: Any
    ) -> AddPipelineRecipientAddPipelineRecipient:
        query = gql(
            """
            mutation AddPipelineRecipient($input: CreatePipelineRecipientInput!) {
              addPipelineRecipient(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="AddPipelineRecipient",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return AddPipelineRecipient.model_validate(data).add_pipeline_recipient

    def delete_pipeline_version(
        self, input: DeletePipelineVersionInput, **kwargs: Any
    ) -> DeletePipelineVersionDeletePipelineVersion:
        query = gql(
            """
            mutation DeletePipelineVersion($input: DeletePipelineVersionInput!) {
              deletePipelineVersion(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="DeletePipelineVersion",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return DeletePipelineVersion.model_validate(data).delete_pipeline_version

    def create_pipeline_template_version(
        self, input: CreatePipelineTemplateVersionInput, **kwargs: Any
    ) -> CreatePipelineTemplateVersionCreatePipelineTemplateVersion:
        query = gql(
            """
            mutation CreatePipelineTemplateVersion($input: CreatePipelineTemplateVersionInput!) {
              createPipelineTemplateVersion(input: $input) {
                success
                errors
                pipelineTemplate {
                  name
                  code
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="CreatePipelineTemplateVersion",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return CreatePipelineTemplateVersion.model_validate(
            data
        ).create_pipeline_template_version

    def delete_pipeline_template(
        self, input: DeletePipelineTemplateInput, **kwargs: Any
    ) -> DeletePipelineTemplateDeletePipelineTemplate:
        query = gql(
            """
            mutation DeletePipelineTemplate($input: DeletePipelineTemplateInput!) {
              deletePipelineTemplate(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="DeletePipelineTemplate",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return DeletePipelineTemplate.model_validate(data).delete_pipeline_template

    def create_pipeline_from_template_version(
        self, input: CreatePipelineFromTemplateVersionInput, **kwargs: Any
    ) -> CreatePipelineFromTemplateVersionCreatePipelineFromTemplateVersion:
        query = gql(
            """
            mutation CreatePipelineFromTemplateVersion($input: CreatePipelineFromTemplateVersionInput!) {
              createPipelineFromTemplateVersion(input: $input) {
                success
                errors
                pipeline {
                  id
                  name
                  code
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="CreatePipelineFromTemplateVersion",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return CreatePipelineFromTemplateVersion.model_validate(
            data
        ).create_pipeline_from_template_version

    def upgrade_pipeline_version_from_template(
        self, input: UpgradePipelineVersionFromTemplateInput, **kwargs: Any
    ) -> UpgradePipelineVersionFromTemplateUpgradePipelineVersionFromTemplate:
        query = gql(
            """
            mutation UpgradePipelineVersionFromTemplate($input: UpgradePipelineVersionFromTemplateInput!) {
              upgradePipelineVersionFromTemplate(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="UpgradePipelineVersionFromTemplate",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return UpgradePipelineVersionFromTemplate.model_validate(
            data
        ).upgrade_pipeline_version_from_template

    def update_webapp(
        self, input: UpdateWebappInput, **kwargs: Any
    ) -> UpdateWebappUpdateWebapp:
        query = gql(
            """
            mutation UpdateWebapp($input: UpdateWebappInput!) {
              updateWebapp(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query, operation_name="UpdateWebapp", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return UpdateWebapp.model_validate(data).update_webapp

    def create_webapp(
        self, input: CreateWebappInput, **kwargs: Any
    ) -> CreateWebappCreateWebapp:
        query = gql(
            """
            mutation CreateWebapp($input: CreateWebappInput!) {
              createWebapp(input: $input) {
                success
                errors
                webapp {
                  id
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query, operation_name="CreateWebapp", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return CreateWebapp.model_validate(data).create_webapp

    def delete_webapp(
        self, input: DeleteWebappInput, **kwargs: Any
    ) -> DeleteWebappDeleteWebapp:
        query = gql(
            """
            mutation DeleteWebapp($input: DeleteWebappInput!) {
              deleteWebapp(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query, operation_name="DeleteWebapp", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return DeleteWebapp.model_validate(data).delete_webapp

    def add_webapp_to_favorites(
        self, input: AddToFavoritesInput, **kwargs: Any
    ) -> AddWebappToFavoritesAddToFavorites:
        query = gql(
            """
            mutation AddWebappToFavorites($input: AddToFavoritesInput!) {
              addToFavorites(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="AddWebappToFavorites",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return AddWebappToFavorites.model_validate(data).add_to_favorites

    def remove_webapp_from_favorites(
        self, input: RemoveFromFavoritesInput, **kwargs: Any
    ) -> RemoveWebappFromFavoritesRemoveFromFavorites:
        query = gql(
            """
            mutation RemoveWebappFromFavorites($input: RemoveFromFavoritesInput!) {
              removeFromFavorites(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="RemoveWebappFromFavorites",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return RemoveWebappFromFavorites.model_validate(data).remove_from_favorites

    def workspace(self, slug: str, **kwargs: Any) -> Optional[WorkspaceWorkspace]:
        query = gql(
            """
            query Workspace($slug: String!) {
              workspace(slug: $slug) {
                slug
                name
                description
                dockerImage
                configuration
                countries {
                  code
                  flag
                  name
                  alpha3
                }
                permissions {
                  delete
                  update
                  manageMembers
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"slug": slug}
        response = self.execute(
            query=query, operation_name="Workspace", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return Workspace.model_validate(data).workspace

    def workspaces(
        self,
        page: Union[Optional[int], UnsetType] = UNSET,
        per_page: Union[Optional[int], UnsetType] = UNSET,
        **kwargs: Any
    ) -> WorkspacesWorkspaces:
        query = gql(
            """
            query Workspaces($page: Int = 1, $perPage: Int = 10) {
              workspaces(page: $page, perPage: $perPage) {
                totalPages
                items {
                  slug
                  name
                  description
                  countries {
                    code
                    alpha3
                    name
                  }
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"page": page, "perPage": per_page}
        response = self.execute(
            query=query, operation_name="Workspaces", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return Workspaces.model_validate(data).workspaces

    def create_workspace(
        self, input: CreateWorkspaceInput, **kwargs: Any
    ) -> CreateWorkspaceCreateWorkspace:
        query = gql(
            """
            mutation CreateWorkspace($input: CreateWorkspaceInput!) {
              createWorkspace(input: $input) {
                success
                workspace {
                  slug
                  name
                  description
                  countries {
                    code
                    alpha3
                    name
                  }
                }
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query, operation_name="CreateWorkspace", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return CreateWorkspace.model_validate(data).create_workspace

    def update_workspace(
        self, input: UpdateWorkspaceInput, **kwargs: Any
    ) -> UpdateWorkspaceUpdateWorkspace:
        query = gql(
            """
            mutation UpdateWorkspace($input: UpdateWorkspaceInput!) {
              updateWorkspace(input: $input) {
                success
                workspace {
                  slug
                  name
                  description
                  configuration
                  countries {
                    code
                    alpha3
                    name
                  }
                }
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query, operation_name="UpdateWorkspace", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return UpdateWorkspace.model_validate(data).update_workspace

    def archive_workspace(
        self, input: ArchiveWorkspaceInput, **kwargs: Any
    ) -> ArchiveWorkspaceArchiveWorkspace:
        query = gql(
            """
            mutation ArchiveWorkspace($input: ArchiveWorkspaceInput!) {
              archiveWorkspace(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="ArchiveWorkspace",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return ArchiveWorkspace.model_validate(data).archive_workspace

    def invite_workspace_member(
        self, input: InviteWorkspaceMemberInput, **kwargs: Any
    ) -> InviteWorkspaceMemberInviteWorkspaceMember:
        query = gql(
            """
            mutation InviteWorkspaceMember($input: InviteWorkspaceMemberInput!) {
              inviteWorkspaceMember(input: $input) {
                success
                errors
                workspaceMembership {
                  id
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="InviteWorkspaceMember",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return InviteWorkspaceMember.model_validate(data).invite_workspace_member

    def create_connection(
        self, input: CreateConnectionInput, **kwargs: Any
    ) -> CreateConnectionCreateConnection:
        query = gql(
            """
            mutation CreateConnection($input: CreateConnectionInput!) {
              createConnection(input: $input) {
                success
                connection {
                  __typename
                  id
                  name
                }
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="CreateConnection",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return CreateConnection.model_validate(data).create_connection

    def update_connection(
        self, input: UpdateConnectionInput, **kwargs: Any
    ) -> UpdateConnectionUpdateConnection:
        query = gql(
            """
            mutation UpdateConnection($input: UpdateConnectionInput!) {
              updateConnection(input: $input) {
                success
                errors
                connection {
                  __typename
                  id
                  name
                  slug
                  description
                  fields {
                    code
                    value
                    secret
                  }
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="UpdateConnection",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return UpdateConnection.model_validate(data).update_connection

    def delete_connection(
        self, input: DeleteConnectionInput, **kwargs: Any
    ) -> DeleteConnectionDeleteConnection:
        query = gql(
            """
            mutation DeleteConnection($input: DeleteConnectionInput!) {
              deleteConnection(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query,
            operation_name="DeleteConnection",
            variables=variables,
            **kwargs
        )
        data = self.get_data(response)
        return DeleteConnection.model_validate(data).delete_connection

    def get_users(
        self, query: str, workspace_slug: str, **kwargs: Any
    ) -> List[GetUsersUsers]:
        _query = gql(
            """
            query GetUsers($query: String!, $workspaceSlug: String!) {
              users(query: $query, workspaceSlug: $workspaceSlug) {
                id
                email
                displayName
                avatar {
                  initials
                  color
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"query": query, "workspaceSlug": workspace_slug}
        response = self.execute(
            query=_query, operation_name="GetUsers", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return GetUsers.model_validate(data).users

    def datasets(
        self,
        query: Union[Optional[str], UnsetType] = UNSET,
        page: Union[Optional[int], UnsetType] = UNSET,
        per_page: Union[Optional[int], UnsetType] = UNSET,
        **kwargs: Any
    ) -> DatasetsDatasets:
        _query = gql(
            """
            query Datasets($query: String, $page: Int = 1, $perPage: Int = 15) {
              datasets(query: $query, page: $page, perPage: $perPage) {
                totalPages
                items {
                  id
                  slug
                  name
                  description
                  createdAt
                  updatedAt
                  createdBy {
                    id
                    displayName
                  }
                  permissions {
                    update
                    delete
                  }
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {
            "query": query,
            "page": page,
            "perPage": per_page,
        }
        response = self.execute(
            query=_query, operation_name="Datasets", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return Datasets.model_validate(data).datasets

    def dataset(self, id: str, **kwargs: Any) -> Optional[DatasetDataset]:
        query = gql(
            """
            query Dataset($id: ID!) {
              dataset(id: $id) {
                id
                slug
                name
                description
                createdAt
                updatedAt
                createdBy {
                  id
                  displayName
                  email
                }
                permissions {
                  update
                  delete
                  createVersion
                }
                versions {
                  items {
                    id
                    name
                    changelog
                    createdAt
                    createdBy {
                      id
                      displayName
                    }
                  }
                }
                workspace {
                  slug
                  name
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"id": id}
        response = self.execute(
            query=query, operation_name="Dataset", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return Dataset.model_validate(data).dataset

    def create_dataset(
        self, input: CreateDatasetInput, **kwargs: Any
    ) -> CreateDatasetCreateDataset:
        query = gql(
            """
            mutation CreateDataset($input: CreateDatasetInput!) {
              createDataset(input: $input) {
                success
                errors
                dataset {
                  slug
                  name
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query, operation_name="CreateDataset", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return CreateDataset.model_validate(data).create_dataset

    def update_dataset(
        self, input: UpdateDatasetInput, **kwargs: Any
    ) -> UpdateDatasetUpdateDataset:
        query = gql(
            """
            mutation UpdateDataset($input: UpdateDatasetInput!) {
              updateDataset(input: $input) {
                success
                errors
                dataset {
                  slug
                  name
                  description
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query, operation_name="UpdateDataset", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return UpdateDataset.model_validate(data).update_dataset

    def delete_dataset(
        self, input: DeleteDatasetInput, **kwargs: Any
    ) -> DeleteDatasetDeleteDataset:
        query = gql(
            """
            mutation DeleteDataset($input: DeleteDatasetInput!) {
              deleteDataset(input: $input) {
                success
                errors
              }
            }
            """
        )
        variables: Dict[str, object] = {"input": input}
        response = self.execute(
            query=query, operation_name="DeleteDataset", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return DeleteDataset.model_validate(data).delete_dataset

    def get_connection(
        self, workspace_slug: str, connection_slug: str, **kwargs: Any
    ) -> Optional[GetConnectionConnectionBySlug]:
        query = gql(
            """
            query getConnection($workspaceSlug: String!, $connectionSlug: String!) {
              connectionBySlug(workspaceSlug: $workspaceSlug, connectionSlug: $connectionSlug) {
                __typename
                type
                fields {
                  code
                  value
                }
              }
            }
            """
        )
        variables: Dict[str, object] = {
            "workspaceSlug": workspace_slug,
            "connectionSlug": connection_slug,
        }
        response = self.execute(
            query=query, operation_name="getConnection", variables=variables, **kwargs
        )
        data = self.get_data(response)
        return GetConnection.model_validate(data).connection_by_slug



# Generated by ariadne-codegen
# Source: openhexa/graphql/queries.graphql

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .enums import DeletePipelineVersionError


class DeletePipelineVersion(BaseModel):
    delete_pipeline_version: "DeletePipelineVersionDeletePipelineVersion" = Field(
        alias="deletePipelineVersion"
    )


class DeletePipelineVersionDeletePipelineVersion(BaseModel):
    success: bool
    errors: List[DeletePipelineVersionError]


DeletePipelineVersion.model_rebuild()
